/*
 * Subroutine to access bt252
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/mm.h>

#define BT252_INTERNAL
#include "bt252.h"
#undef  BT252_INTERNAL
#include "fgrab.h"
#include "fgrab_pci.h"
#include "saa7116.h"

struct bt252 *bt252[MAXFGRABS+1] ;

static void setup(struct bt252 *bt, unsigned irq,unsigned minor) ;
static void init(unsigned minor) ;
static void load_ad_lut(unsigned minor, unsigned index, unsigned char val) ;
static unsigned get_ad_lut(unsigned minor, unsigned index) ;
static void set_white_level(unsigned minor, unsigned char val) ;
static unsigned char get_white_level(unsigned minor) ;
static void set_black_level(unsigned minor, unsigned char val) ;
static unsigned char get_black_level(unsigned minor) ;
static void select_video_src(unsigned minor, unsigned channel) ;
static void select_sync_src(unsigned minor, unsigned channel) ;
static void set_sync_det_level(unsigned minor, unsigned level) ;

struct bt252_progs bt252_progs = {
    setup,
    init,
    load_ad_lut, get_ad_lut,
    set_white_level, get_white_level, set_black_level, get_black_level,
    select_video_src,
    select_sync_src, set_sync_det_level
} ;

static struct symbol_table bt252_syms = {
#include <linux/symtab_begin.h>
    X(bt252_progs),
#include <linux/symtab_end.h>
} ;    

/* Designed to be used as a module */
#ifdef MODULE

void cleanup_module(void) {
}


int init_module(void) {
    register_symtab(&bt252_syms);
    return(0);
}
#endif /*MODULE*/

static void
setup(struct bt252 *bt, unsigned irq,unsigned minor)
{
    bt252[minor] = bt ;
    bt->minor = minor ;
}

static void load_ad_lut(unsigned minor, unsigned index, unsigned char val) {
    saa7116_progs.talk_i2c(minor,_IIC_W(BT252_AD_ADDR), 0, (unsigned char)index) ;
    saa7116_progs.talk_i2c(minor,_IIC_W(BT252_AD_LUT), 0, val) ;
}

static unsigned get_ad_lut(unsigned minor, unsigned index) {
    saa7116_progs.talk_i2c(minor,_IIC_W(BT252_AD_ADDR), 0, (unsigned char)index) ;
    saa7116_progs.talk_i2c(minor,_IIC_R(BT252_AD_LUT), 0, 0) ;
    return(saa7116_progs.get_i2c_data(minor));
}

static void set_white_level(unsigned minor, unsigned char val) {
    saa7116_progs.talk_i2c(minor,_IIC_W(BT252_AD_ADDR), 0, BT252_ADDR_IOUT0) ;
    saa7116_progs.talk_i2c(minor,_IIC_W(BT252_AD_CMD), 0, val) ;
}

static unsigned char get_white_level(unsigned minor) {
    saa7116_progs.talk_i2c(minor,_IIC_W(BT252_AD_ADDR), 0, BT252_ADDR_IOUT0) ;
    saa7116_progs.talk_i2c(minor,_IIC_R(BT252_AD_CMD), 0, 0) ;
    return((unsigned char)saa7116_progs.get_i2c_data(minor));
}

static void set_black_level(unsigned minor, unsigned char val) {
    saa7116_progs.talk_i2c(minor,_IIC_W(BT252_AD_ADDR), 0, BT252_ADDR_IOUT1) ;
    saa7116_progs.talk_i2c(minor,_IIC_W(BT252_AD_CMD), 0, val) ;
}

static unsigned char get_black_level(unsigned minor) {
    saa7116_progs.talk_i2c(minor,_IIC_W(BT252_AD_ADDR), 0, BT252_ADDR_IOUT1) ;
    saa7116_progs.talk_i2c(minor,_IIC_R(BT252_AD_CMD), 0, 0) ;
    return((unsigned char)saa7116_progs.get_i2c_data(minor));
}

static unsigned char get_bt_cmd(unsigned minor) {
    saa7116_progs.talk_i2c(minor,_IIC_W(BT252_AD_ADDR), 0, BT252_ADDR_CMD) ;
    saa7116_progs.talk_i2c(minor,_IIC_R(BT252_AD_CMD), 0, 0) ;
    return ((unsigned char)saa7116_progs.get_i2c_data(minor)) ;
}

static void put_bt_cmd(unsigned minor, unsigned char bc) {
    saa7116_progs.talk_i2c(minor,_IIC_W(BT252_AD_CMD), 0, (unsigned)bc) ;
}

static void init(unsigned minor) {
    int i;
    put_bt_cmd(minor,0);
    for (i=0;i<256;i++) load_ad_lut(minor,i,i);
}

static void select_video_src(unsigned minor, unsigned channel) {
    put_bt_cmd(minor,BT252_DIGITIZE_CNL_SEL_W(get_bt_cmd(minor),channel));
}

static void select_sync_src(unsigned minor, unsigned channel) {
    put_bt_cmd(minor,BT252_SYNC_CNL_SEL_W(get_bt_cmd(minor),channel));
}

static void set_sync_det_level(unsigned minor, unsigned level) {
    put_bt_cmd(minor,BT252_SYNC_LEVEL_W(get_bt_cmd(minor),level));
}


/*
 * Following is from drivers/scsi/hosts.c of 2.0.18.
 * Overrides for Emacs so that we follow Linus's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-indent-level: 4
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -4
 * c-argdecl-indent: 4
 * c-label-offset: -4
 * c-continued-statement-offset: 4
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */
