/*
 * Definitions for bt252 chip
 */

#ifdef    BT252_INTERNAL
#define BT252_AD_ADDR	0x30
#define BT252_AD_LUT	0x31
#define BT252_AD_CMD	0x32
/* SYNC_LEVEL: 0-3 ==> 50, 75, 100, 125 mv  */
#       define BT252_SYNC_LEVEL_R(arg)           ((arg>>2)&0x3)
#       define BT252_SYNC_LEVEL_W(old,arg)       ( ((old) &~(0x3<<2)) | (((arg)&0x3)<<2) )
#       define BT252_SYNC_CNL_SEL_R(arg)         ((arg>>4)&0x3)
#       define BT252_SYNC_CNL_SEL_W(old,arg)     ( ((old) &~(0x3<<4)) | (((arg)&0x3)<<4) )
#       define BT252_DIGITIZE_CNL_SEL_R(arg)     ((arg>>6)&0x3)
#       define BT252_DIGITIZE_CNL_SEL_W(old,arg) ( ((old) &~(0x3<<6)) | (((arg)&0x3)<<6) )

/* Values of AD_ADDR register that affect function of AD_CMD register   */
#define BT252_ADDR_CMD   0x0
#define BT252_ADDR_IOUT0 0x1
#define BT252_ADDR_IOUT1 0x2

struct bt_cmd {
    unsigned :2;                        /* unused                       */
    unsigned sync_level:2;              /* 0-3 ==> 50, 75, 100, 125 mv  */
    unsigned sync_select:2;             /* select sync source           */
    unsigned digitize_select:2;         /* select video input (0-3)     */
} ;

#endif /* BT252_INTERNAL */

struct bt252 {
    unsigned minor;
} ;

struct bt252_progs {
    void (*setup)(struct bt252 *bt, unsigned irq,unsigned minor) ;
    void (*init)(unsigned minor) ;
    void (*load_ad_lut)(unsigned minor, unsigned index, unsigned char val) ;
    unsigned (*get_ad_lut)(unsigned minor, unsigned index) ;
    void (*set_white_level)(unsigned minor, unsigned char val) ;
    unsigned char (*get_white_level)(unsigned minor) ;
    void (*set_black_level)(unsigned minor, unsigned char val) ;
    unsigned char (*get_black_level)(unsigned minor) ;
    void (*select_video_src)(unsigned minor, unsigned channel) ;
    void (*select_sync_src)(unsigned minor, unsigned channel) ;
    void (*set_sync_det_level)(unsigned minor, unsigned level) ;
} ;

#ifndef   BT252_INTERNAL
extern struct bt252_progs bt252_progs ;
#endif /* BT252_INTERNAL */

/*
 * Following is from drivers/scsi/hosts.c of 2.0.18.
 * Overrides for Emacs so that we follow Linus's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-indent-level: 4
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -4
 * c-argdecl-indent: 4
 * c-label-offset: -4
 * c-continued-statement-offset: 4
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */
