/*    bt254 - routines for communicating with bt254 through saa7116
 *    
 *    See file COPYING for copyright info
 *
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/mm.h>

#define BT254_INTERNAL
#include "bt254.h"
#undef  BT254_INTERNAL
#include "fgrab.h"
#include "fgrab_pci.h"
#include "saa7116.h"

struct bt254 *bt254[MAXFGRABS+1] ;

static void setup(struct bt254 *bt, unsigned irq,unsigned minor) ;
static int init(unsigned minor) ;
static void select_saa7196(unsigned minor) ;
static void select_bt254(unsigned minor) ;
static void ntsc(unsigned minor, int arg) ;
static void bt254_write(unsigned minor, u_char addr, u_char data) ;
static unsigned get_reg(unsigned minor, unsigned arg_val) ;


struct bt254_progs bt254_progs = {
    setup, init, select_saa7196, select_bt254, ntsc, bt254_write, get_reg
} ;

static struct symbol_table bt254_syms = {
#include <linux/symtab_begin.h>
    X(bt254_progs),
#include <linux/symtab_end.h>
} ;    

/* Designed to be used as a module */
#ifdef MODULE

void cleanup_module(void) {
}


int init_module(void) {
    register_symtab(&bt254_syms);
    return(0);
}
#endif /*MODULE*/



static u_char bt254_default[NUM_BT254_REGS] = {
	0x00, 	/* 24 bpp */
	0xa0,
	0xa0,
	0xa0,
	0x50,
	0x50,
	0x50,
} ;


static void
bt254_write(unsigned minor, u_char addr, u_char data) {
    addr &= 0x7;						/* sanity? */
    bt254[minor]->bt254_reg[addr] = data;
    PCF8574_DATA_WRITE(minor, data);				/* set data */
    PCF8574_CTRL_WRITE(minor, (PCF8574_CTRL_REG(minor) & ~0x7) | addr);
    PCF8574_CTRL_WRITE(minor, PCF8574_CTRL_REG(minor) & ~0x10);	/* WR/ to 0 */
    PCF8574_CTRL_WRITE(minor, PCF8574_CTRL_REG(minor) | 0x10);	/* WR to 1 */
    PCF8574_DATA_WRITE(minor, 0xff);				/* clr data */
}

static unsigned
get_reg(unsigned minor, unsigned arg_val) {
    arg_val &= 0x7;
    return((bt254[minor]->bt254_reg[arg_val])<<4 | arg_val) ; 
}

static void
rgb_init(unsigned minor) {
    int	i;

    PCF8574_CTRL_WRITE(minor, 0x7f);
    PCF8574_DATA_WRITE(minor, 0xff);	/* data port must be 0xff */
    PCF8574_CTRL_WRITE(minor, 0x7f);
    
    /* init RGB module for 24bpp, composite input */
    for(i=0; i<NUM_BT254_REGS; i++)
        bt254_write(minor, i, bt254_default[i]);
    
    bt254_write(minor, BT254_COMMAND, 0x00);	/* 24 bpp */
}

static int
init(unsigned minor) {
    int rgb;

    /*
     * Check for RGB module, initialized if found.
     */
    if(saa7116_progs.talk_i2c(minor,PCF8574_DATA_I2C_ADDR_W,0,0xff) == 0) {
        PRINTK((KERN_INFO "fgrab%d: <Booktree 254 (RGB module)>\n",
                minor));	/* does this have a rev #? */
        rgb_init(minor);	/* Set up RGB module */
        rgb = 1 ;
    } else {
        PRINTK((KERN_INFO
                "fgrab%d: i2c abort normal if no RGB module\n", minor)) ;
        rgb = 0;
    }
    return(rgb) ;
}       

static void
ntsc(unsigned minor, int arg) {
    if (arg){
        /* Set NTSC bit */
        PCF8574_CTRL_WRITE(minor, PCF8574_CTRL_REG(minor) | 0x20);
    }
    else {
        /* reset NTSC bit */
        PCF8574_CTRL_WRITE(minor, PCF8574_CTRL_REG(minor) &= ~0x20);
    }
}

static void
select_bt254(unsigned minor)
{

	/* disable saa7196, saaen = 1 */
	PCF8574_CTRL_WRITE(minor, PCF8574_CTRL_REG(minor) | 0x80);
	/* enable Bt254, bten = 0 */
	PCF8574_CTRL_WRITE(minor, PCF8574_CTRL_REG(minor) & ~0x40);
}

static void
select_saa7196(unsigned minor)
{
	/* disable Bt254, bten = 1 */
	PCF8574_CTRL_WRITE(minor, PCF8574_CTRL_REG(minor) | 0x40);
	/* enable saa7196, saaen = 0 */
	PCF8574_CTRL_WRITE(minor, PCF8574_CTRL_REG(minor) & ~0x80);
}



/*
 * Initialize the RGB module.
 */
static void
setup(struct bt254 *bt, unsigned irq,unsigned minor)
{
    bt254[minor] = bt ;
    bt->minor = minor ;
}

/*
 * Following is from drivers/scsi/hosts.c of 2.0.18.
 * Overrides for Emacs so that we follow Linus's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-indent-level: 4
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -4
 * c-argdecl-indent: 4
 * c-label-offset: -4
 * c-continued-statement-offset: 4
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */
