/*    bt254 include file
 *    
 *    See file COPYING for copyright info
 *
 */

/*
 * Defines for the PCF8574.
 *
 * There are two PCF8574As on the board.  One is at address 0x70 
 * (W: 0x70, R: 0x71) and performs various control functions to do with 
 * the RGB section:
 *
 * P7: saa7196 enable (0=en, 1=dis)
 * P6: bt254 enable (0=en, 1=dis)
 * P5: ntsc (1=ntsc, 0=pal/secam)
 * P4: BT254 WR*
 * P3: ?? (probably BT254 RD*)
 * P2: BT254 A2
 * P1: BT254 A1
 * P0: BT254 A0
 *
 * The other is the data for (or from) the Bt254 at address 0x72 (W: 0x72, R: 0x73)
 *
 */
#define NUM_PCF8574_I2C_REGS	2
#define	PCF8574_CTRL_I2C_ADDR_W	0x70
#define	PCF8574_CTRL_I2C_ADDR_R	0x71
#define PCF8574_DATA_I2C_ADDR_W	0x72
#define PCF8574_DATA_I2C_ADDR_R	0x73
#define	PCF8574_CTRL_WRITE(minor, data)                                 \
    saa7116_progs.talk_i2c(minor,                                       \
                           PCF8574_CTRL_I2C_ADDR_W, data, data), 	\
	bt254[minor]->pcf_i2c[0] = data
#define	PCF8574_DATA_WRITE(minor, data) 				\
    saa7116_progs.talk_i2c(minor,                                       \
                           PCF8574_DATA_I2C_ADDR_W, data, data), 	\
	bt254[minor]->pcf_i2c[1] = data
#define PCF8574_CTRL_REG(minor) bt254[minor]->pcf_i2c[0]
#define PCF8574_DATA_REG(minor) bt254[minor]->pcf_i2c[1]


/*
 * Defines for the BT254.
 */
#define	NUM_BT254_REGS	7

#define BT254_COMMAND	0
#define	BT254_IOUT1	1
#define	BT254_IOUT2	2
#define	BT254_IOUT3	3
#define BT254_IOUT4	4
#define	BT254_IOUT5	5
#define	BT254_IOUT6	6

struct bt254 {
    struct saa7116 *saa7116;
    u_char	pcf_i2c[NUM_PCF8574_I2C_REGS];	  /* PCF8574 register values */
    u_char	bt254_reg[NUM_BT254_REGS];	  /* BT254 register values   */
    unsigned int minor;
} ;

struct bt254_progs {
    void (*setup)(struct bt254 *bt, unsigned irq,unsigned minor) ;
    int  (*init)(unsigned minor);
    void (*select_saa7196)(unsigned minor);
    void (*select_bt254)(unsigned minor);
    void (*ntsc)(unsigned minor, int arg);
    void (*write)(unsigned minor, u_char addr, u_char data) ;
    unsigned (*get_reg)(unsigned minor, unsigned arg_val) ;
} ;

#ifndef    BT254_INTERNAL
extern struct bt254_progs bt254_progs;
#endif /* BT254_INTERNAL */




/*
 * Following is from drivers/scsi/hosts.c of 2.0.18.
 * Overrides for Emacs so that we follow Linus's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-indent-level: 4
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -4
 * c-argdecl-indent: 4
 * c-label-offset: -4
 * c-continued-statement-offset: 4
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */
