/*    dt315x.c  -- DT315[25] board specific code for Generic Frame Grabber.
 *    
 *    See file COPYING for copyright info
 *
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/mm.h>
#include <linux/delay.h>

#include "fgrab.h"
#include "fgrab_pci.h"
#include "saa7116.h"
#include "bt252.h"
#include "dt315x.h"
#include "dt315x_ioctl.h"

/*
 * dt315x_detect is the only externally known symbol
 * It must be included in the module symbol table.
 */
int dt315x_detect(void) ;
static struct symbol_table dt315x_syms = {
#include <linux/symtab_begin.h>
    X(dt315x_detect),
#include <linux/symtab_end.h>
} ;    


#if defined(PAL)
#define DT315X_DEFAULT_PAL
#define DT315X_MAXFPS 25
#elif defined(SECAM)
#define DT315X_DEFAULT_SECAM
#define DT315X_MAXFPS 25
#else
#define DT315X_DEFAULT_NTSC
#define DT315X_MAXFPS 30
#endif

static struct dt315x *dt315x[MAXFGRABS];

static struct fgrab_progs dt3152_progs, dt3155_progs ;

static int dt315x_register(unsigned int irq,struct saa7116 *saa7116) ;
static int dt315x_signature(void) ;
static void unwind(unsigned minor) ;
static void dt315x_init(unsigned minor) ;
static void open(unsigned minor) ;
static void close(unsigned minor) ;
static void start_capture(unsigned minor, unsigned single_flag) ;
static void stop_capture(unsigned minor) ;
static int set_fps(unsigned minor, u_short fps) ;
static void saa7116_init(unsigned minor) ;
static void saa7116_reset(unsigned minor) ;
static void dt3152_set_video_gain(unsigned minor, unsigned gain) ;
static unsigned dt3152_get_video_gain(unsigned minor) ;
static void set_brightness(unsigned minor, u_long brightness) ;
static unsigned get_brightness(unsigned minor) ;
static void set_contrast(unsigned minor, u_long contrast) ;
static unsigned get_contrast(unsigned minor) ;
static void set_vertical_freq(unsigned minor) ;
static void set_pixel_clock(unsigned minor, u_long freq) ;
static unsigned get_pixel_clock(unsigned minor) ;
static void set_hor_wndw_strt(unsigned minor,unsigned long hor_wndw_strt) ;
static unsigned get_hor_wndw_strt(unsigned minor) ;
static void set_vert_wndw_strt(unsigned minor,unsigned long vert_wndw_strt) ;
static unsigned get_vert_wndw_strt(unsigned minor) ;
static void set_xclip(unsigned minor) ;
static void set_yclip(unsigned minor) ;
static void set_rows(unsigned minor, unsigned rows) ;
static void set_cols(unsigned minor, unsigned cols) ;
static void set_fmt_ram_mask(unsigned minor, unsigned sel_vert, 
                             unsigned index, unsigned mask) ;
static void set_format(unsigned minor) ;
static int set_input_dev_0(unsigned minor) ;
static int set_input_dev_1(unsigned minor) ;
static int set_input_dev_2(unsigned minor) ;
static int set_input_dev_3(unsigned minor) ;
static void set_input_ext_sync(unsigned minor, unsigned long flag) ;
static int set_fmt_ntsc(unsigned minor) ;
static int set_fmt_pal(unsigned minor) ;
static int set_fmt_secam(unsigned minor) ;
static int set_field_odd_only(unsigned minor) ;
static int set_field_even_only(unsigned minor) ;
static int set_field_interlace(unsigned minor) ;
static int set_field_alternate(unsigned minor) ;
static int set_field_non_interlace(unsigned minor) ;
static int set_frame(unsigned minor, int rows, int cols, int frames) ;
static int set_geo_mono8(unsigned minor) ;
static int ioctl(unsigned minor, unsigned cmd, u_long *arg_val,int *write_flag) ;
static int ckioctl(unsigned minor, unsigned ioctl, unsigned function) ;



static void setup_progs(void) {
    dt3152_progs.open                   = open ;
    dt3152_progs.close                  = close ;
    dt3152_progs.unwind                 = unwind ;
    dt3152_progs.set_dma_e              = saa7116_progs.set_dma_e ;
    dt3152_progs.set_dma_o              = saa7116_progs.set_dma_o ;
    dt3152_progs.enable_dma             = fgrab_pci_progs.enable_dma ;
    dt3152_progs.disable_dma            = fgrab_pci_progs.disable_dma ;
    dt3152_progs.start_capture          = start_capture ;
    dt3152_progs.stop_capture           = stop_capture ;
    dt3152_progs.check_intr             = saa7116_progs.check_intr ;
    dt3152_progs.enable_intr            = saa7116_progs.enable_intr ;
    dt3152_progs.disable_intr           = saa7116_progs.disable_intr ;
    dt3152_progs.check_err              = saa7116_progs.check_err ;
    dt3152_progs.clear_err              = saa7116_progs.clear_err ;
    dt3152_progs.set_fps                = set_fps ;
    dt3152_progs.set_hue                = NULL ;
    dt3152_progs.get_hue                = NULL ;
    dt3152_progs.set_chroma_gain        = NULL ;
    dt3152_progs.get_chroma_gain        = NULL ;
    dt3152_progs.set_brightness         = set_brightness ;
    dt3152_progs.get_brightness         = get_brightness ;
    dt3152_progs.set_chroma_saturation  = NULL ;
    dt3152_progs.get_chroma_saturation  = NULL ;
    dt3152_progs.set_contrast           = set_contrast ;
    dt3152_progs.get_contrast           = get_contrast  ;
    dt3152_progs.set_hor_wndw_strt      = set_hor_wndw_strt ;
    dt3152_progs.get_hor_wndw_strt      = get_hor_wndw_strt ;
    dt3152_progs.set_vert_wndw_strt     = set_vert_wndw_strt ;
    dt3152_progs.get_vert_wndw_strt     = get_vert_wndw_strt ;
    dt3152_progs.set_pixel_clock        = set_pixel_clock ;
    dt3152_progs.get_pixel_clock        = get_pixel_clock ;
    dt3152_progs.set_input_dev_0        = set_input_dev_0 ;
    dt3152_progs.set_input_dev_1        = set_input_dev_1 ;
    dt3152_progs.set_input_dev_2        = set_input_dev_2 ;
    dt3152_progs.set_input_dev_3        = set_input_dev_3 ;
    dt3152_progs.set_input_dev_svideo   = NULL ;
    dt3152_progs.set_input_dev_rgb      = NULL ;
    dt3152_progs.set_input_ext_sync     = set_input_ext_sync ;
    dt3152_progs.set_fmt_ntsc           = set_fmt_ntsc ;
    dt3152_progs.set_fmt_pal            = set_fmt_pal ;
    dt3152_progs.set_fmt_secam          = set_fmt_secam ;
    dt3152_progs.set_fmt_auto           = NULL ;
    dt3152_progs.set_field_odd_only     = set_field_odd_only ;
    dt3152_progs.set_field_even_only    = set_field_even_only ;
    dt3152_progs.set_field_interlace    = set_field_interlace ;
    dt3152_progs.set_field_alternate    = set_field_alternate ;
    dt3152_progs.set_field_non_interlace= set_field_non_interlace ;
    dt3152_progs.set_frame              = set_frame ;
    dt3152_progs.set_geo_rgb16          = NULL ;
    dt3152_progs.set_geo_rgb24          = NULL ;
    dt3152_progs.set_geo_mono8          = set_geo_mono8 ;
    dt3152_progs.set_geo_yuv_9          = NULL ;
    dt3152_progs.set_geo_yuv_12         = NULL ;
    dt3152_progs.set_geo_yuv_16         = NULL ;
    dt3152_progs.set_geo_yuv_packed     = NULL ;
    dt3152_progs.set_chroma_signed      = NULL ;
    dt3152_progs.ioctl                  = ioctl ;      
    dt3152_progs.ckioctl                = ckioctl ;      
    /*
     * dt3155 is like 3152 except missing programable pixel clock
     * and no external sync (slow scan) capability.
     */
    dt3155_progs                        = dt3152_progs;
    dt3155_progs.set_pixel_clock        = NULL ;
    dt3155_progs.get_pixel_clock        = NULL ;
    dt3152_progs.set_input_ext_sync     = NULL ;
}


struct dt315x_fmt dt315x_ntsc = { 
    horz_active:    {125, 765},
    horz_reset:     770,
    horz_insert:    800,
    video_clamp:    {93, 95}, 
    field_det:      {200, 600},
    pixel_clock:    12500000,
    vert_active:    {10,250},
    vert_reset:     253,
    vert_insert:    265,
    vfreq:          60
} ;

struct dt315x_fmt dt315x_pal = { 
    horz_active:    {160, 928},
    horz_reset:     864,
    horz_insert:    1022,
    video_clamp:    {90, 93}, 
    field_det:      {240, 720},
    pixel_clock:    15000000,
    vert_active:    {12,268},
    vert_reset:     271,
    vert_insert:    320,
    vfreq:          50
} ;



/* Designed to be used as a module */
#ifdef MODULE
void cleanup_module(void) {
    unsigned minor;
    for (minor=0;minor<MAXFGRABS;minor++) {
        if (dt315x[minor] != NULL) {
            unwind(minor);
        }
    }
}
int init_module(void) {
    register_symtab(&dt315x_syms);
    return(dt315x_detect());
}
#endif /*MODULE*/

static void dt315x_setup(void) {
    int i;
    for (i=0;i<MAXFGRABS;i++) dt315x[i] = NULL ;
    setup_progs();
}

/*
 * Detect presence of boards.
 * Initialize the boards, data structures, etc.
 */
int dt315x_detect(void) {
    int err;
    dt315x_setup();
    PRINTK((KERN_INFO "dt315x: ready to call saa7116 detect routine\n"));
    err=saa7116_progs.detect(dt315x_register,dt315x_signature);
    return err;
}

/*
 * dt_model is used durring detection-registration sequence
 * it is referenced only by dt315x_signature and dt315x_register
 */
static int dt_model;    
/* signature determines whether a particular 7116 board is a
   dt315x board or some other 7116 based board                              */
static int dt315x_signature(void) {
    saa7116_init(FGRAB_MINOR_SETUP);
    /*
     * Try communicating with DT ID register.
     * If it fails then this is clearly not a DT board
     */
    if (saa7116_progs.talk_i2c(FGRAB_MINOR_SETUP,_IIC_R(DT315X_DT_ID),0,0)==1) return(0);
    /*
     * If the above worked then lets see what value is there.
     */
    switch(saa7116_progs.get_i2c_data(FGRAB_MINOR_SETUP)) {
    case DT3152_ID:
        dt_model = 3152 ;
        return(1);
    case DT3155_ID:
        dt_model = 3155 ;
        return(1);
    default:
        return(0);
    }
}

static int dt315x_register(unsigned int irq,struct saa7116 *saa7116) {
    int err;
    struct dt315x *dt;
    unsigned int minor;
    if ((err=fgrab_register(irq,
                            (dt_model==3155)?&dt3155_progs:&dt3152_progs,
                            sizeof(struct dt315x),
                            (void *)&dt,&minor))) {
        printk(KERN_ERR "fgrab: dt315x: cannot register for irq %d\n", irq);
        return(err);
    }
    dt315x[minor] = dt ;
    dt->minor = minor ;
    dt->irq = irq ;
    dt->model = dt_model;       /* this was set in signature    */
    dt->saa7116 = *saa7116;
    saa7116_progs.setup(&(dt->saa7116),irq,minor);
    bt252_progs.setup(&(dt->bt252),irq,minor);
    dt315x_init(minor);
    printk(KERN_INFO "Dt315x driver version " DT315X_VERSION 
           ": minor %d board DT%04d ready\n", minor,(int)dt->model);
    return(0);
}

/*
 * Initialize the 7116, 7196 and the RGB module.
 */
static void
dt315x_init(unsigned minor)
{
  /*
   * Initialize the Philips SAA7116
   */
    saa7116_init(minor);
    saa7116_progs.probe_range(minor);
    set_fps(minor, DT315X_MAXFPS);
}

/* open is the operations that need to be done when a process 
   claims the device                                                    */
/* reinitialize device registers                                        */
	/*
	 * Make sure that the i2c regs are set the same for each open.
	 */
static void
open(unsigned minor) {
    int i;
    MOD_INC_USE_COUNT;
    dt315x_init(minor);
    dt315x[minor]->flags = 0 ;
    saa7116_progs.talk_i2c(minor,_IIC_W(DT315X_EVEN_CSR),0,0);
    saa7116_progs.talk_i2c(minor,_IIC_W(DT315X_ODD_CSR),0,0);
    saa7116_progs.talk_i2c(minor,_IIC_W(DT315X_CONFIG),0,0);
    saa7116_progs.talk_i2c(minor,_IIC_W(DT315X_DIG_OUT),0,0);
    if (dt315x[minor]->model==3155) {
        saa7116_progs.talk_i2c(minor,_IIC_W(DT3155_CSR2),0,DT3155_CSR2_SYNC_SNTL);
    } else {    /* 3152 */
        saa7116_progs.talk_i2c(minor,_IIC_W(DT3152_ACQ_CSR0),0,
                               /* Leaving out sync sentinel gives lock on first frame RWF 4jul97        */
                               (0&DT3152_CSR0_SYNC_SNTL)   |
                               /* Setting sel0 and sel1 chosses DC coupling at a-to-d input
                                  but there is still a capicitor at the front end RWF 4jul97            */
                               DT3152_CSR0_FILTER_SEL0 |
                               DT3152_CSR0_FILTER_SEL1   );
        saa7116_progs.talk_i2c(minor,_IIC_W(DT3152_ACQ_CSR4),0,0);
        saa7116_progs.talk_i2c(minor,_IIC_W(DT3152_XY_INCREMENT),0,0);
        saa7116_progs.talk_i2c(minor,_IIC_W(DT3152_DTC),0,0);
    }
    bt252_progs.init(minor);
    if (dt315x[minor]->model==3152) {
        unsigned gain  = 1 ; /* 1 is unity gain (range is 2**(n-1) for nfrom 0 to 3)    */
        dt3152_set_video_gain(minor, gain) ;
    }
    set_brightness(minor,0x80); /* 0x80 was default of meteor   */
    set_contrast(minor,0x40);   /* 0x40 was default of meteor   */
    saa7116_progs.talk_i2c(minor,_IIC_W(DT315X_PM_LUT_ADDR),0,0);
    for (i=0;i<256;i++) saa7116_progs.talk_i2c(minor,_IIC_W(DT315X_PM_LUT_DATA),0,i);
    dt315x[minor]->fmt = 
#       ifdef DT315X_DEFAULT_NTSC
            &dt315x_ntsc;
#       else
            &dt315x_pal;
#       endif
    dt315x[minor]->hor_wndw_strt = dt315x[minor]->fmt->horz_active[0];
    dt315x[minor]->cols = dt315x[minor]->fmt->horz_active[1] - 
                          dt315x[minor]->fmt->horz_active[0] ;
    dt315x[minor]->vert_wndw_strt = dt315x[minor]->fmt->vert_active[0];
    dt315x[minor]->rows = dt315x[minor]->fmt->vert_active[1] - 
                          dt315x[minor]->fmt->vert_active[0] ;
    if (dt315x[minor]->model==3155) {
        set_yclip(minor);
        set_xclip(minor);
    } else {
        set_format(minor) ;
        set_pixel_clock(minor,dt315x[minor]->fmt->pixel_clock);
    }
    set_vertical_freq(minor);
}

static void
close(unsigned minor)
{
    saa7116_reset(minor);
    MOD_DEC_USE_COUNT;
}

/*
 * These routines with name prefixes of saa7116_ are at the board level
 * because they have delays that seem to vary from board to board.
 */

/*
 * Initialize the saa7116.
 * The incantations for this seem to differ from board to board RWF
 */

static void saa7116_init(unsigned minor) {
    saa7116_reset(minor);
    /* capt_ctl = 0x000080c0  Bring out of reset */
    /* Note dt documetation omits RANGE_EN RWF   */
    saa7116_progs.set_capt_ctl(minor, 
                               SAA7116_CSR1_RANGE_EN    | 
                               SAA7116_CSR1_FIFO_EN     |
                               SAA7116_CSR1_SRST );
    fgrab_wait_jiffies(DT315X_WAIT_JIFFS);
    saa7116_progs.set_i2c_clocks(minor, 0x01, 0x01, 0x01, 0x01) ;       /* 0x01010101 */
    saa7116_progs.set_geo_rgb16(minor);
    saa7116_progs.set_fifo_trigger(minor, SAA7116_PACKED ) ;
    saa7116_progs.disable_field_toggle(minor) ;
    saa7116_progs.reset_int_flags(minor) ;      /* Clear and disable all interrupts */
    /* This combination of masks and mask length means capture all fields */
    saa7116_progs.set_field_masks(minor, 0x00000001, 0, 0x00000001, 0) ;
    saa7116_progs.set_fifo_limits(minor) ;
    fgrab_wait_jiffies(DT315X_WAIT_JIFFS);	/* Let this sink in */
}

/*
 * Reset the saa7116.
 */
static void saa7116_reset(unsigned minor) {
    fgrab_wait_jiffies(DT315X_WAIT_JIFFS);		/* Let things settle down      */
    /* capt_ctl = 0x00000f30  Reset */
    saa7116_progs.set_capt_ctl(minor, 
                               SAA7116_CSR1_ADDR_ERR_ODD        | 
                               SAA7116_CSR1_ADDR_ERR_EVE        | 
                               SAA7116_CSR1_FLD_CRPT_ODD        | 
                               SAA7116_CSR1_FLD_CRPT_EVE        | 
                               SAA7116_CSR1_FLD_DN_ODD          | 
                               SAA7116_CSR1_FLD_DN_EVE          );
    fgrab_wait_jiffies(DT315X_WAIT_JIFFS);		/* Let things settle down      */
    saa7116_progs.set_capt_ctl(minor, 0x00000000) ;     /* First, put into reset       */
    saa7116_progs.zero_regs(minor) ;                    /* Clear and reset everything  */
    fgrab_wait_jiffies(DT315X_WAIT_JIFFS);		/* Let the dust settle         */
}

static void start_capture(unsigned minor, unsigned single_flag) {
    if (dt315x[minor]->model==3152) {
        saa7116_progs.talk_i2c(minor,_IIC_R(DT3152_ACQ_CSR0),0,0);
        saa7116_progs.talk_i2c(minor,_IIC_W(DT3152_ACQ_CSR0),0,
            (saa7116_progs.get_i2c_data(minor) & ~(DT3152_CSR0_BUSY_ODD|DT3152_CSR0_BUSY_EVE) )
            | ( (dt315x[minor]->flags&DT315X_ONLY_EVEN_FIELDS) ? 0 : DT3152_CSR0_BUSY_ODD )
            | ( (dt315x[minor]->flags&DT315X_ONLY_ODD_FIELDS ) ? 0 : DT3152_CSR0_BUSY_EVE )
            );
    } else {    /* 3155 */
        saa7116_progs.talk_i2c(minor,_IIC_R(DT3155_CSR2),0,0);
        saa7116_progs.talk_i2c(minor,_IIC_W(DT3155_CSR2),0,
            (saa7116_progs.get_i2c_data(minor) & ~(DT3155_CSR2_BUSY_ODD|DT3155_CSR2_BUSY_EVE) )
            | ( (dt315x[minor]->flags&DT315X_ONLY_EVEN_FIELDS) ? 0 : DT3155_CSR2_BUSY_ODD )
            | ( (dt315x[minor]->flags&DT315X_ONLY_ODD_FIELDS ) ? 0 : DT3155_CSR2_BUSY_EVE )
            );
    }
    saa7116_progs.start_capture(minor,single_flag);
}

static void stop_capture(unsigned minor) {
    saa7116_progs.stop_capture(minor);
}

static int 
set_fps(unsigned minor, u_short fps)
{
    int maxfps, mask, length;
    int b, step;

    if (fps<1)
	return (-1);

    maxfps=DT315X_MAXFPS ;
    if ( (fps > maxfps) || (fps < 1) ) return(-1) ;
    if (fps == maxfps) {
        mask = 0x1;
        length = 0x0;
    } else if (2*fps == maxfps) {
        mask = 0x1;
        length = 0x1;
    } else if (fps > maxfps/2) {
        int den = maxfps - fps ;
        mask = (1<<maxfps) - 1;
        length = maxfps - 1;
        step = (maxfps - 1) ;
        for (b=step; b<maxfps*den; b+=step) {
            mask &= ~(1<<(b/den));  /* mask out the bth frame */
        }
    } else {
        mask = 0x1;
        length = maxfps - 1;
        step = (maxfps - 1) ;
        for (b=step+fps; b<(maxfps-1)*fps; b+=step) {
            mask |= (1<<(b/fps));  /* mask in the bth frame */
        }
    }

    saa7116_progs.set_field_masks(minor,mask,length,mask,length);
    return 0;
}

static void unwind(unsigned minor) {
    saa7116_reset(minor);
    saa7116_progs.unwind(minor) ;
    fgrab_unregister(minor);
}

static void dt3152_set_video_gain(unsigned minor, unsigned gain) {
    unsigned csr;
    saa7116_progs.talk_i2c(minor,_IIC_R(DT3152_ACQ_CSR4),0,0);
    csr = saa7116_progs.get_i2c_data(minor);
    saa7116_progs.talk_i2c(minor,_IIC_W(DT3152_ACQ_CSR4),0,
                           (csr&~DT3152_CSR4_GAIN_MASK) | (gain&DT3152_CSR4_GAIN_MASK) );
}
static unsigned dt3152_get_video_gain(unsigned minor) {
    saa7116_progs.talk_i2c(minor,_IIC_R(DT3152_ACQ_CSR4),0,0);
    return(DT3152_CSR4_GAIN_MASK & saa7116_progs.get_i2c_data(minor));
}

/*
 * The meteor uses brightness and contrast to control analog gain and offset
 * ahead of the A/D.  The dt boards use a bt252 chip which is documented to
 * set the black and white levels (min and max values) that the A/D uses.  
 * However, it appears that dt has chosen to use that chip a little differently.
 * The BT252 has two d-to-a's on it that can be connected to the a-to-d black
 * and white level terminals so that their registers (documented as 6 bits in bt252
 * documentation but billed as 8 bits in dt documentation) would then set black
 * and white.  I believe dt has used those d-to-a outputs differently.  It appears
 * that the one nominally used for black level is instead used as a bias so that
 * it effectively sets brightness.  The black setting of the a-to-d is then tied 
 * to some fixed level (perhaps ground) so that the second d-to-a effectively sets 
 * contrast.  The dt board also has a gain control that moves in powers of 2 through
 * 4 levels.  That will be initialized to unity since that gives a reasonable
 * range of sensitivities (excursions from 0 to 255 of digitized data can occur
 * with input analog data excursions of .24v at the higest contrast to 1.52v at
 * the lowest contrast setting.  There is an additional DT specific ioctl that
 * can be used to change the gain but since that gain control is unique to the
 * dt such software will not be protable.  Another option would be to make the
 * contrast variable more than 8 bits.  Then if the dt board is thought of as
 * using 12 bits the gain could be set by shifting the contrast the number of
 * bits to the right to make it so that only the lower 8 bits are non-zero.
 * The number of bits shifted would then determine the gain setting and the 
 * remaining 8 bits would go to the d-to-a that sets contrast.  That hasn't
 * been done in order to maintain compatibility with meteor software but there
 * will come a time to relax that constraint to let this driver grow with
 * the technology of the hardware.  RWF 4jul97 
 */
static void set_black_white(unsigned minor) {
    int black, white;
    black = (dt315x[minor]->brightness) ;
    black = (black<0) ? 0 : ( (black>0xff) ? 0xff : black ) ;
    white = 255 - dt315x[minor]->contrast ;
    white = (white<0) ? 0 : ( (white>0xff) ? 0xff : white ) ;
    bt252_progs.set_black_level(minor,(unsigned char)black);
    bt252_progs.set_white_level(minor,(unsigned char)white);
}
static void set_brightness(unsigned minor,unsigned long brightness) {
    dt315x[minor]->brightness = brightness;
    set_black_white(minor);
}

static unsigned get_brightness(unsigned minor) {
    unsigned brightness;
    unsigned char black = bt252_progs.get_black_level(minor);
    brightness = (black<<2);
    return(brightness);
}

static void set_contrast(unsigned minor,unsigned long contrast) {
    dt315x[minor]->contrast = contrast;
    set_black_white(minor);
}

static unsigned get_contrast(unsigned minor) {
    unsigned white = (int)bt252_progs.get_white_level(minor);
    unsigned contrast; 
    contrast = 255 - white ;
    return(contrast);
}


/* reverse_bits is used by the pixel_clock routines     */
static unsigned reverse_bits(unsigned in) {
    unsigned out;
    int i;
    out = 0;
    for (i=0;i<8;i++) {
        out = out<<1 | (in&0x1) ;
        in >>= 1 ;
    }
    return(out);
}
        
static void set_pixel_clock(unsigned minor, unsigned long freq) {
    unsigned div, pcnt, rcnt, ncnt, factor, fp, fvcoem1;
    int del, min_del, i;
    unsigned min_ncnt=0, min_rcnt=0, min_pcnt=0;
    unsigned char bval[6];
    if (freq>20000000) freq = 20000000;
    if (freq<1000) freq = 1000 ;
    for (div=0,factor=2;( freq*factor ) <= 20000000;div++,factor <<= 1) {
        if (div==0xf) break;
    }
    fp = freq*factor;
    if (fp<20000000) fp = 20000000 ;
    min_del = -1 ;
    for (pcnt=((130000000/fp)-1);
	 pcnt>=((65000000+fp-1)/fp)-1;
	 pcnt--) {
        fvcoem1 = (fp*(pcnt+1))/10 ;
        for (rcnt=28;rcnt<=50;rcnt++) {
            for (ncnt=(fvcoem1*rcnt)/2000000;ncnt<=((fvcoem1+1)*rcnt)/2000000;ncnt++) {
                del = (2000000*ncnt)/(rcnt*(pcnt+1))-fp ;
                del = (del<0) ? -del : del ;
                if ( (del < min_del) || (min_del < 0) ) {
                    min_ncnt = ncnt ;
                    min_rcnt = rcnt ;
                    min_pcnt = pcnt ;
                    min_del  = del  ;
                }
            }
        }
    }
    bval[0] = 0xd4 ;
#   define DP8534_RESET 0x10
    bval[1] = ((min_pcnt&0x1)<<7) | 0x20 ;
    bval[2] = ((min_pcnt>>1)&0x7) | ((min_rcnt&0x1)<<7) ;
    bval[3] = (min_rcnt>>1)&0xff ;
    bval[4] = ((min_rcnt>>9)&0x1) | ((min_ncnt&0x7f)<<1) ;
    bval[5] = ((min_ncnt>>7)&0x7f) | 0x80 ;
    for (i=0;i<=4;i++) {
        saa7116_progs.talk_i2c(minor,_IIC_W(DT3152_CLOCK1),0,reverse_bits(bval[i]));
    }
    saa7116_progs.talk_i2c(minor,_IIC_W(DT3152_CLOCK2),0,reverse_bits(bval[5]));
    saa7116_progs.talk_i2c(minor,_IIC_W(DT3152_CLK_DIVIDER),0,(DT3152_CLK_DIV&div) );
}

static void set_vertical_freq(unsigned minor) {
    if (dt315x[minor]->model==3155) {
        saa7116_progs.talk_i2c(minor,_IIC_R(DT3155_CSR2),0,0);
        saa7116_progs.talk_i2c(minor,_IIC_W(DT3155_CSR2),0,
                 (saa7116_progs.get_i2c_data(minor)&~DT3155_CSR2_50HZ) 
                 | (dt315x[minor]->fmt->vfreq==50)?DT3155_CSR2_50HZ:0    );
    } else {
        /* If we use chroma filters this needs to be filled in  */
    }
}

static unsigned get_pixel_clock(unsigned minor) {
    unsigned div, pcnt, rcnt, ncnt, factor, freq;
    int i;
    unsigned char bval[6];
    saa7116_progs.talk_i2c(minor,_IIC_R(DT3152_CLOCK2),0,0);
    bval[0] = reverse_bits(saa7116_progs.get_i2c_data(minor));
    for (i=1;i<=5;i++) {
        saa7116_progs.talk_i2c(minor,_IIC_R(DT3152_CLOCK1),0,0);
        bval[i] = reverse_bits(saa7116_progs.get_i2c_data(minor));
    }
    pcnt = ((bval[1]>>7)&0x1) | ((bval[2]&0x7)<<1) ;
    rcnt = ((bval[2]>>7)&0x1) | ((bval[3]&0xff)<<1) | ((bval[4]&0x1)<<9) ;
    ncnt = ((bval[4]>>1)&0x7f) | ((bval[5]&0x7f)<<7) ;
    saa7116_progs.talk_i2c(minor,_IIC_R(DT3152_CLK_DIVIDER),0,0);
    div = DT3152_CLK_DIV & saa7116_progs.get_i2c_data(minor) ;
    factor = (0x2)<<div ;
    freq = ((2000000*ncnt)/(rcnt*(pcnt+1)*factor))*10 ;
    return(freq);
}

static unsigned get_hor_wndw_strt(unsigned minor) {
    return(dt315x[minor]->hor_wndw_strt);
}

static unsigned get_vert_wndw_strt(unsigned minor) {
    return(dt315x[minor]->vert_wndw_strt);
}

static void set_hor_wndw_strt(unsigned minor,unsigned long hor_wndw_strt) {
    dt315x[minor]->hor_wndw_strt = hor_wndw_strt ;
    if (dt315x[minor]->model==3155) set_xclip(minor) ;
    else set_format(minor);
}

static void set_vert_wndw_strt(unsigned minor,unsigned long vert_wndw_strt) {
    dt315x[minor]->vert_wndw_strt = vert_wndw_strt ;
    if (dt315x[minor]->model==3155) set_yclip(minor) ;
    else set_format(minor);
}

/* The set clip routines are specific to the dt3155                     */
static void set_xclip(unsigned minor) {
    saa7116_progs.talk_i2c(minor,_IIC_W(DT3155_X_CLIP_START),0,
             (dt315x[minor]->hor_wndw_strt)>>2);
    saa7116_progs.talk_i2c(minor,_IIC_W(DT3155_X_CLIP_END),0,
             (dt315x[minor]->hor_wndw_strt+dt315x[minor]->cols)>>2);
}
static void set_yclip(unsigned minor) {
    saa7116_progs.talk_i2c(minor,_IIC_W(DT3155_Y_CLIP_START),0,
             (dt315x[minor]->vert_wndw_strt)>>2);
    saa7116_progs.talk_i2c(minor,_IIC_W(DT3155_Y_CLIP_END),0,
             (dt315x[minor]->vert_wndw_strt+dt315x[minor]->rows)>>2);
}

static void set_rows(unsigned minor, unsigned rows) {
    dt315x[minor]->rows = rows ;
    if (dt315x[minor]->model==3155) set_yclip(minor);
    else set_format(minor);
}

static void set_cols(unsigned minor, unsigned cols) {
    dt315x[minor]->cols = cols ;
    if (dt315x[minor]->model==3155) set_xclip(minor);
    else set_format(minor);
}

/* this routine is used exclusively by set_format to set a bit in fmt ram       */
static void set_fmt_ram_mask(unsigned minor, unsigned sel_vert, 
                             unsigned index, unsigned mask) {
    unsigned old;
    /* Write bottom 8 bits of index */
    saa7116_progs.talk_i2c(minor,_IIC_W(DT3152_FMT_ADDR_L),0,index&0xff);     
    /* Write top 4 bits of index along with flag to select vertical     */
    saa7116_progs.talk_i2c(minor,_IIC_W(DT3152_FMT_ADDR_H),0,
             ((index>>8)&0xf) | ((sel_vert!=0)?DT3152_FMT_SEL_VERT:0)  );
    /* Get previous mask                                                */
    saa7116_progs.talk_i2c(minor,_IIC_R(DT3152_FMT_DATA_ACCESS),0,0);
    old = saa7116_progs.get_i2c_data(minor);
    /* Write bottom 8 bits of index */
    saa7116_progs.talk_i2c(minor,_IIC_W(DT3152_FMT_ADDR_L),0,index&0xff);     
    /* Write top 4 bits of index along with flag to select vertical     */
    saa7116_progs.talk_i2c(minor,_IIC_W(DT3152_FMT_ADDR_H),0,
             ((index>>8)&0xf) | ((sel_vert!=0)?DT3152_FMT_SEL_VERT:0)  );
    /* Or mask in with old mask                                         */
    saa7116_progs.talk_i2c(minor,_IIC_W(DT3152_FMT_DATA_ACCESS),0,old|mask);
}
/* Routine to get mask elements from format ram       */
static unsigned char get_fmt_ram_mask(unsigned minor, unsigned sel_vert, unsigned index) {
    /* Write bottom 8 bits of index */
    saa7116_progs.talk_i2c(minor,_IIC_W(DT3152_FMT_ADDR_L),0,index&0xff);     
    /* Write top 4 bits of index along with flag to select vertical     */
    saa7116_progs.talk_i2c(minor,_IIC_W(DT3152_FMT_ADDR_H),0,
             ((index>>8)&0xf) | ((sel_vert!=0)?DT3152_FMT_SEL_VERT:0)  );
    /* Get mask */
    saa7116_progs.talk_i2c(minor,_IIC_R(DT3152_FMT_DATA_ACCESS),0,0);
    return(saa7116_progs.get_i2c_data(minor));
}
/*
 * DT3152 specifc routine:
 * This routine sets the format ram to give the appropriate number of pixels
 * and rows.  It should be called whenever the horizontal or vertical
 * offsets or lengths are changed or the underlying video format (ntsc vs pal)
 * is changed.
 */
static void set_format(unsigned minor) {
    struct dt315x *dt=dt315x[minor] ;
    int jifcnt;
    /* Clear entire format memory       */
    saa7116_progs.talk_i2c(minor,_IIC_W(DT3152_MASK),0,DT3152_MASK_BOTH);
    /* Wait till flush is done          */
    udelay(70);
    for (jifcnt=0;;jifcnt++) {  
        saa7116_progs.talk_i2c(minor,_IIC_R(DT3152_ACQ_CSR4),0,0) ;
        if ( (DT3152_CSR4_FLUSH & saa7116_progs.get_i2c_data(minor)) == 0)
            break;
        /* clear of both banks should take 820 us       */
        if (jifcnt > FGRAB_WAIT_MS2JIF(3)) {
            PRINTK((KERN_INFO "fgrab%d: timeout error durring clear format memory\n", minor));
            return;
        }
        fgrab_wait_jiffies(1);
    }
    set_fmt_ram_mask(minor,0,dt->hor_wndw_strt,           DT3152_FMT_ACTIVE     );
    set_fmt_ram_mask(minor,0,dt->hor_wndw_strt+dt->cols,  DT3152_FMT_ACTIVE     );
    set_fmt_ram_mask(minor,0,dt->fmt->horz_reset,         DT3152_FMT_RESET      );
    set_fmt_ram_mask(minor,0,dt->fmt->horz_insert,        DT3152_FMT_INSERT     );
    set_fmt_ram_mask(minor,0,dt->fmt->video_clamp[0],     DT3152_FMT_VIDEO_CLAMP);
    set_fmt_ram_mask(minor,0,dt->fmt->video_clamp[1],     DT3152_FMT_VIDEO_CLAMP);
    set_fmt_ram_mask(minor,0,dt->fmt->field_det[0],       DT3152_FMT_FIELD_DET  );
    set_fmt_ram_mask(minor,0,dt->fmt->field_det[1],       DT3152_FMT_FIELD_DET  );
    set_fmt_ram_mask(minor,1,dt->vert_wndw_strt,          DT3152_FMT_ACTIVE     );
    set_fmt_ram_mask(minor,1,dt->vert_wndw_strt+dt->rows, DT3152_FMT_ACTIVE     );
    set_fmt_ram_mask(minor,1,dt->fmt->vert_reset,         DT3152_FMT_RESET      );
    set_fmt_ram_mask(minor,1,dt->fmt->vert_insert,        DT3152_FMT_INSERT     );
}


#define SETPROG(name) static int set_##name(unsigned minor)
#define GETPROG(name) static unsigned int get_##name(unsigned minor) 
#define SETPROGNA(name) static int set_##name(unsigned minor)

SETPROGNA(input_dev_0) { 
    bt252_progs.select_video_src(minor,0);
    bt252_progs.select_sync_src(minor,0);
    return(0);
}

SETPROGNA(input_dev_1) { 
    bt252_progs.select_video_src(minor,1);
    bt252_progs.select_sync_src(minor,1);
    return(0);
}

SETPROGNA(input_dev_2) { 
    bt252_progs.select_video_src(minor,2);
    bt252_progs.select_sync_src(minor,2);
    return(0);
}

SETPROGNA(input_dev_3) { 
    bt252_progs.select_video_src(minor,3);
    bt252_progs.select_sync_src(minor,3);
    return(0);
}

SETPROGNA(fmt_ntsc) { 
    dt315x[minor]->fmt = &dt315x_ntsc ;
    if (dt315x[minor]->model==3152) set_format(minor) ;
    set_vertical_freq(minor);
    return(0);
}

SETPROGNA(fmt_pal) { 
    dt315x[minor]->fmt = &dt315x_pal ;
    if (dt315x[minor]->model==3152) set_format(minor) ;
    set_vertical_freq(minor);
    return(0);
}

SETPROGNA(fmt_secam) { 
    dt315x[minor]->fmt = &dt315x_pal ;
    if (dt315x[minor]->model==3152) set_format(minor) ;
    set_vertical_freq(minor);
    return(0);
}

/* dt3152 routine for external sync (so called slow scan)       */
static void set_input_ext_sync(unsigned minor, unsigned long flag) {
    unsigned char csr4;
    saa7116_progs.talk_i2c(minor,_IIC_R(DT3152_ACQ_CSR4),0,0);
    csr4 = saa7116_progs.get_i2c_data(minor);
    csr4 = (csr4&~DT3152_CSR4_SLOW_SCAN) | 
           ( (flag) ? DT3152_CSR4_SLOW_SCAN : 0 ) ;
    saa7116_progs.talk_i2c(minor,_IIC_W(DT3152_ACQ_CSR4),0,csr4);
}

static int set_field_odd_only(unsigned minor) {
    struct dt315x *dt = dt315x[minor] ;
    dt->flags = (dt->flags & ~DT315X_FIELDS_MASK) 
        | DT315X_ONLY_ODD_FIELDS ;
    saa7116_progs.set_field_odd_only(minor);
    /*
     * This code uses defines for the bits of this register
     * which are the same for the 3152 and 3155
     */
    saa7116_progs.talk_i2c(minor,_IIC_R(DT315X_CONFIG),0,0);
    saa7116_progs.talk_i2c(minor,_IIC_W(DT315X_CONFIG),0,
        (saa7116_progs.get_i2c_data(minor)&~DT315X_CONFIG_ACQ_MODE_MASK)
         |DT315X_CONFIG_ACQ_MODE_ODD);
    return(0);
}
static int set_field_even_only(unsigned minor) {
    struct dt315x *dt = dt315x[minor] ;
    dt->flags = (dt->flags & ~DT315X_FIELDS_MASK) 
        | DT315X_ONLY_EVEN_FIELDS ;
    saa7116_progs.set_field_even_only(minor);
    saa7116_progs.talk_i2c(minor,_IIC_R(DT315X_CONFIG),0,0);
    saa7116_progs.talk_i2c(minor,_IIC_W(DT315X_CONFIG),0,
        (saa7116_progs.get_i2c_data(minor)&~DT315X_CONFIG_ACQ_MODE_MASK)
         |DT315X_CONFIG_ACQ_MODE_EVEN);
    return(0);
}
static int set_field_alternate(unsigned minor) {
    struct dt315x *dt = dt315x[minor] ;
    dt->flags = (dt->flags & ~DT315X_FIELDS_MASK) 
        | DT315X_ALTERNATE_FIELDS ;
    saa7116_progs.set_field_both(minor);
    saa7116_progs.talk_i2c(minor,_IIC_R(DT315X_CONFIG),0,0);
    saa7116_progs.talk_i2c(minor,_IIC_W(DT315X_CONFIG),0,
        (saa7116_progs.get_i2c_data(minor)&~DT315X_CONFIG_ACQ_MODE_MASK)
         |DT315X_CONFIG_ACQ_MODE_EVEN);
    return(0);
}
static int set_field_interlace(unsigned minor) {
    struct dt315x *dt = dt315x[minor] ;
    dt->flags = (dt->flags & ~DT315X_FIELDS_MASK) 
        | DT315X_INTERLACE_FIELDS ;
    saa7116_progs.set_field_both(minor);
    /*
     * According to dt docs this should cause start on even field
     */
    saa7116_progs.talk_i2c(minor,_IIC_R(DT315X_CONFIG),0,0);
    saa7116_progs.talk_i2c(minor,_IIC_W(DT315X_CONFIG),0,
        (saa7116_progs.get_i2c_data(minor)&~DT315X_CONFIG_ACQ_MODE_MASK)
         |DT315X_CONFIG_ACQ_MODE_EVEN|( (dt->model==3152)?DT3152_CSR3_ELINE_1ST:0 ) );
    return(0);
}
    
static int set_field_non_interlace(unsigned minor) {
    struct dt315x *dt = dt315x[minor] ;
    dt->flags = (dt->flags & ~DT315X_FIELDS_MASK) 
        | DT315X_NON_INTERLACE_FIELDS ;
    saa7116_progs.set_field_both(minor);
    /*
     * According to dt docs this should be used to non-intelaced data
     */
    saa7116_progs.talk_i2c(minor,_IIC_R(DT315X_CONFIG),0,0);
    saa7116_progs.talk_i2c(minor,_IIC_W(DT315X_CONFIG),0,
        (saa7116_progs.get_i2c_data(minor)&~DT315X_CONFIG_ACQ_MODE_MASK)
         |DT315X_CONFIG_ACQ_MODE_NON_INTERLACED);
    return(0);
}
    

static int set_frame(unsigned minor, int rows, int cols, int frames) {
    struct dt315x *dt = dt315x[minor] ;
    int error=0;

    /* The dma address increments in 4 byte steps, which means dma_end will
     * only be correct if columns%4 == 0.  For YUV we require columns%8 == 0
     * since for each 8 bits of Y there are 4 bits each for U and V.  This bit
     * of code takes the conservative option.  In field mode mode rows don't
     * need to be even, but we're conservative here again.  In field mode the 
     * number of fields specified (geo->frames) must be even since the board 
     * will only stop capture after odd fields.
     */
    if ((cols & 0x3f8) != cols) {
        PRINTK((KERN_INFO
                "fgrab%ld: dt315x: %d: columns too large or not diviside by 8.\n",
                dt->minor, cols));
        error = -EINVAL;
    }
    if (((rows & 0x7fe) != rows) ||
        (( (dt->flags & DT315X_FIELDS_MASK) == DT315X_ALTERNATE_FIELDS ) &&
         ((rows & 0x3fe) != rows)) ) {
	PRINTK((KERN_INFO
		"fgrab%ld: dt315x: %d: rows too large or not even.\n",
		dt->minor, rows));
	error = -EINVAL;
    }
    if (((dt->flags & DT315X_FIELDS_MASK)==DT315X_ALTERNATE_FIELDS) && 
        (frames > 1) && 
        (frames & 0x1) == 0x1) {
	PRINTK((KERN_INFO
		"fgrab%ld: dt315x: %d: must have an even number of fields in field mode.\n",
		dt->minor, frames));
	error = -EINVAL;
      }
    if (error!=0) return error;

    /* set cols */
    set_cols(minor,cols);
    /* set rows */
    switch(dt->flags&DT315X_FIELDS_MASK) {
    case DT315X_ONLY_ODD_FIELDS:
    case DT315X_ONLY_EVEN_FIELDS:
    case DT315X_ALTERNATE_FIELDS:
    case DT315X_NON_INTERLACE_FIELDS:
        set_rows(minor,rows);
        break;
    case DT315X_INTERLACE_FIELDS:
        set_rows(minor,rows>>1);
        break;
    }
    return(0);
}

/* These are guesses RWF 21JUN97        */
SETPROG(geo_mono8) { 
    saa7116_progs.set_geo_rgb16(minor);
    return(0);
}


static int 
ioctl(unsigned minor, unsigned cmd, u_long *arg_val,int *write_flag) {
    int	error = 0 ;  

    switch (cmd) {

    case DT3152SGEXP:	/* set the dt3152 gain exponent */
    {
        if (dt315x[minor]->model!=3152) {
            error = -EINVAL ;
            break;
        }
        dt3152_set_video_gain(minor,*arg_val) ;
        break;
    }
    case DT3152GGEXP:	/* get the dt3152 gain exponent */
    {
        if (dt315x[minor]->model!=3152) {
            error = -EINVAL ;
            break;
        }
        *arg_val = dt3152_get_video_gain(minor) ;
        *write_flag = 1 ;
        break;
    }
    case DT315XGCAPT:	/* get capt_ctl */
    {
        *arg_val = saa7116_progs.get_capt_ctl(minor) ;
        *write_flag = 1 ;
        break;
    }
    case DT3152GCSR4:
    {
        if (dt315x[minor]->model!=3152) {
            error = -EINVAL ;
            break;
        }
        saa7116_progs.talk_i2c(minor,_IIC_R(DT3152_ACQ_CSR4),0,0);
        *arg_val = saa7116_progs.get_i2c_data(minor) ;
        *write_flag = 1 ;
        break;
    }
    case DT3152GHFMTMEM:
    {
        unsigned index = (unsigned) *arg_val ;
        if (dt315x[minor]->model!=3152) {
            error = -EINVAL ;
            break;
        }
        *arg_val = (unsigned long) get_fmt_ram_mask(minor, 0, index) ;
        *write_flag = 1 ;
        break;
    }
    case DT3152GVFMTMEM:
    {
        unsigned index = (unsigned) *arg_val ;
        if (dt315x[minor]->model!=3152) {
            error = -EINVAL ;
            break;
        }
        *arg_val = (unsigned long) get_fmt_ram_mask(minor, 1, index) ;
        *write_flag = 1 ;
        break;
    }
    case DT3152GADLUT:
    {
        unsigned index = (unsigned) *arg_val ;
        if (dt315x[minor]->model!=3152) {
            error = -EINVAL ;
            break;
        }
        *arg_val = (unsigned long) bt252_progs.get_ad_lut(minor, index) ;
        *write_flag = 1 ;
        break;
    }
    case DT3152GPMLUT:
    {
        unsigned index = (unsigned) *arg_val ;
        if (dt315x[minor]->model!=3152) {
            error = -EINVAL ;
            break;
        }
        saa7116_progs.talk_i2c(minor,_IIC_W(DT315X_PM_LUT_ADDR),0,index);
        saa7116_progs.talk_i2c(minor,_IIC_R(DT315X_PM_LUT_DATA),0,0);
        *arg_val = (unsigned long) saa7116_progs.get_i2c_data(minor) ;
        *write_flag = 1 ;
        break;
    }
    default:
        PRINTK((KERN_INFO "fgrab%d: ioctl: invalid ioctl request %x\n", minor, cmd));
        error = -ENOTTY;
        break;
    }  
    return(error);

}

static int ckioctl(unsigned minor, unsigned ioctl, unsigned function) {
    int error = 0 ;
    switch(ioctl) {
    case DT315XGCAPT:	/* get capt_ctl */
        break;
    default:
        error = -ENOTTY;
        break;
    }  
    return(error);
}


/*
 * Following is from drivers/scsi/hosts.c of 2.0.18.
 * Overrides for Emacs so that we follow Linus's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-indent-level: 4
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -4
 * c-argdecl-indent: 4
 * c-label-offset: -4
 * c-continued-statement-offset: 4
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */
