/*    dt315x.h  -- DT315[25] board specific include file of Generic Frame Grabber.
 *    
 *    See file COPYING for copyright info
 *
 */

/*
 * I2C offsets of registers
 * Note the convention that registers that are the same for both boards
 * start with DT315X while registers that are specific to one or the other
 * start with DT3152 or DT3155
 */

#define DT3152_ACQ_CSR0		0x10
#	define DT3152_CSR0_SYNC_MASTER	(0x1<<7)
#	define DT3152_CSR0_SYNC_PRESENT	(0x1<<6)
#	define DT3152_CSR0_SYNC_SNTL	(0x1<<5)
#	define DT3152_CSR0_FILTER_SEL1	(0x1<<4)
#	define DT3152_CSR0_FILTER_SEL0	(0x1<<3)
#	define DT3152_CSR0_DISP_PASS	(0x1<<2)
#	define DT3152_CSR0_BUSY_ODD	(0x1<<1)
#	define DT3152_CSR0_BUSY_EVE	(0x1<<0)
#define DT3155_CSR2		0x10
#	define DT3155_CSR2_DISP_PASS	(0x1<<6)
#	define DT3155_CSR2_BUSY_ODD	(0x1<<5)
#	define DT3155_CSR2_BUSY_EVE	(0x1<<4)
#	define DT3155_CSR2_SYNC_PRESENT	(0x1<<3)
#	define DT3155_CSR2_50HZ		(0x1<<2)
#	define DT3155_CSR2_SYNC_SNTL	(0x1<<1)
#	define DT3155_CSR2_CHROM_FIL	(0x1<<0)

#define DT315X_EVEN_CSR		0x11
#define DT315X_ODD_CSR		0x12
#	define DT315X_CSR_ERROR	(0x1<<2)
#	define DT315X_CSR_SNGL	(0x1<<1)
#	define DT315X_CSR_DONE	(0x1<<0)

#define DT3152_ACQ_CSR3		0x13
#	define DT3152_CSR3_ELINE_1ST	(0x1<<5)
#	define DT3152_CSR3_LUT_SEL	(0x1<<4)
#	define DT3152_CSR3_EXT_TRIG_POL	(0x1<<3)
#	define DT3152_CSR3_EXT_TRIG_EN	(0x1<<2)
#	define DT3152_CSR3_ACQ_MODE_MASK	0x3
#	define DT3152_CSR3_ACQ_MODE_EVEN		0x0
#	define DT3152_CSR3_ACQ_MODE_ODD			0x1
#	define DT3152_CSR3_ACQ_MODE_NEXT		0x2
#	define DT3152_CSR3_ACQ_MODE_NON_INTERLACED	0x3
#define DT3155_CONFIG		0x13
#	define DT3155_CONFIG_PM_LUT_PGM		(0x1<<7)
#	define DT3155_CONFIG_PM_LUT_SEL		(0x1<<6)
#	define DT3155_CONFIG_CLIP		(0x1<<5)
#	define DT3155_CONFIG_H_SCALE		(0x1<<4)
#	define DT3155_CONFIG_EXT_TRIG_POL	(0x1<<3)
#	define DT3155_CONFIG_EXT_TRIG_EN	(0x1<<2)
#	define DT3155_CONFIG_ACQ_MODE_MASK	0x3
#	define DT3155_CONFIG_ACQ_MODE_EVEN		0x0
#	define DT3155_CONFIG_ACQ_MODE_ODD		0x1
#	define DT3155_CONFIG_ACQ_MODE_NEXT		0x2
#	define DT3155_CONFIG_ACQ_MODE_NON_INTERLACED	0x3
/* 
 * Parts of the iic registers at offset 0x13 are the same for the 3152 and 3155.
 * The following provides a common method of accessing those parts
*/
#define DT315X_CONFIG		0x13
#	define DT315X_CONFIG_EXT_TRIG_POL	(0x1<<3)
#	define DT315X_CONFIG_EXT_TRIG_EN	(0x1<<2)
#	define DT315X_CONFIG_ACQ_MODE_MASK	0x3
#	define DT315X_CONFIG_ACQ_MODE_EVEN		0x0
#	define DT315X_CONFIG_ACQ_MODE_ODD		0x1
#	define DT315X_CONFIG_ACQ_MODE_NEXT		0x2
#	define DT315X_CONFIG_ACQ_MODE_NON_INTERLACED	0x3

#define DT3152_ACQ_CSR4		0x14
#	define DT3152_CSR4_SLOW_SCAN		(0x1<<7)
#	define DT3152_CSR4_FLUSH		(0x1<<6)
#	define DT3152_CSR4_FRAME_ERROR		(0x1<<5)
#	define DT3152_CSR4_LINE_ERROR		(0x1<<4)
#	define DT3152_CSR4_FRAME_SYNC_INV	(0x1<<3)
#	define DT3152_CSR4_LINE_SYNC_INV	(0x1<<2)
#	define DT3152_CSR4_GAIN_SEL1		(0x1<<1)
#	define DT3152_CSR4_GAIN_SEL0		(0x1<<0)
#	define DT3152_CSR4_GAIN_MASK		(DT3152_CSR4_GAIN_SEL1|DT3152_CSR4_GAIN_SEL0)
#define DT3152_XY_INCREMENT	0x15
struct dt3152_xy_increment { unsigned yinc:4; unsigned xinc:4; } ;
/* Following 4 locations deal with the pixel layout as well as master sync generation	*/
#define DT3152_FMT_ADDR_L	0x18
#define DT3152_FMT_ADDR_H	0x19
#	define DT3152_FMT_SEL_VERT	(0x1<<4)
#define DT3152_FMT_DATA_ACCESS	0x1a
#	define DT3152_FMT_ACTIVE	(0x1<<0)
#	define DT3152_FMT_RESET		(0x1<<1)
#	define DT3152_FMT_INSERT	(0x1<<2)
#	define DT3152_FMT_VIDEO_CLAMP	(0x1<<3)	/* Horizontal only	*/
#	define DT3152_FMT_FIELD_DET	(0x1<<4)	/* Horizontal only	*/
#	define DT3152_FMT_ACT_DTC	(0x1<<5)
#define DT3152_MASK		0x1c
#	define DT3152_MASK_HRAM		(0x0<<6)
#	define DT3152_MASK_VRAM		(0x1<<6)
#	define DT3152_MASK_BOTH		(0x2<<6)

#define DT315X_DT_ID		0x1f
#define DT3152_ID			0x2a /* value of DT_ID for dt3152	*/
#define DT3155_ID			0x20 /* value of DT_ID for dt3155	*/

#define DT3155_X_CLIP_START	0x20
#define DT3155_Y_CLIP_START	0x22
#define DT3155_X_CLIP_END	0x24
#define DT3155_Y_CLIP_END	0x26

/* 
 * Note that the BT252 registers AD_ADDR, AD_LUT, and AD_CMD
 * occupy 0x30, 0x31, and 0x32 respectively.  See bt252.h
 */

#define DT3152_CLOCK1		0x34	/* write bytes 0-4, read bytes 1-5	*/
#define DT3152_CLOCK2		0x35	/* write byte 5, read byte 0		*/
#define DT3152_CLK_DIVIDER	0x36
#	define DT3152_CLK_DIV 		0x0f
#	define DT3152_CLK_EX_CLK_INV    0x10
#	define DT3152_CLK_CLK_MODE      0x20
#define DT3152_DTC		0x38
struct dt3152_dtc { 
    unsigned dtc_enable:1;
    unsigned dtc_error:1;
    unsigned even_strt:1;
    unsigned sngl_frame:1;
    unsigned sync_to_acq:1;
    unsigned inc:3;
} ;

#define DT315X_DIG_OUT		0x40
#define DT315X_PM_LUT_ADDR	0x50
#define DT315X_PM_LUT_DATA	0x51


#define DT315X_FIELDS_MASK  		0x07000000
#define DT315X_INTERLACE_FIELDS		0x00000000
#define DT315X_ONLY_EVEN_FIELDS		0x01000000
#define DT315X_ONLY_ODD_FIELDS		0x02000000
#define DT315X_ALTERNATE_FIELDS		0x03000000
#define DT315X_NON_INTERLACE_FIELDS	0x04000000


/* Should be at least 1 millisecond	*/
#define DT315X_WAIT_MS 1
#define DT315X_WAIT_JIFFS FGRAB_WAIT_MS2JIF(DT315X_WAIT_MS)

struct dt315x_fmt {
    /*
     * horz_active and vert_active are init settings for window
     * The rest of the parameters are permanent.
     */
    unsigned horz_active[2];
    unsigned horz_reset;
    unsigned horz_insert;
    unsigned video_clamp[2];
    unsigned field_det[2];
    unsigned pixel_clock;
    unsigned vert_active[2];
    unsigned vert_reset;
    unsigned vert_insert;
    unsigned vfreq;
} ;

struct dt315x	{
    struct saa7116 saa7116;	/* structure private to saa7116 code  	*/
    struct bt252   bt252  ;	/* structure private to bt252 code  	*/
    struct dt315x_fmt *fmt;	/* pointer to format structure		*/
    u_short hor_wndw_strt;	/* start of horizontal window		*/
    u_short vert_wndw_strt;	/* start of vertical window		*/
    u_short cols;		/* number of active columns		*/
    u_short rows;		/* number of active rows		*/
    u_char irq;			/* IRQ 					*/
    u_long minor;		/* minor device number for this board	*/
    u_long flags;		/* a couple of flags			*/
    u_long model;		/* 3155 or 3152 to flag which board	*/
    u_long brightness;		/* brightness requested			*/
    u_long contrast;		/* contrast requested			*/
    unsigned bus_errors;	/* cumulative bus erors since open	*/
    unsigned ext_bus_errors;	/* cumulative external erors since open	*/
    } ;

#define DT315X_VERSION "0.1"


