/*    fgrab.h -- include file for Generic Frame Grabber.
 *    
 *    See file COPYING for copyright info
 *
 */

/* General-Purpose #defines */

#define FGRAB_VERSION "0.01"

/* Get rid of DEBUG_FGRAB once the driver works for you. */
/*#define DEBUG_FGRAB */

#define FGRAB_MAJOR 40

#define MAXFGRABS 2

#define	FGRAB_INITALIZED	0x00000001
#define	FGRAB_OPEN		0x00000002 
#define	FGRAB_MMAP		0x00000004
#define	FGRAB_INTR		0x00000008
#define	FGRAB_ONCE		0x00000010	/* capture all frames once */
#define	FGRAB_SINGLE		0x00000020	/* get single frame */
#define	FGRAB_CONTIN		0x00000040	/* continuously get frames */
#define	FGRAB_SYNCAP		0x00000080	/* synchronously get frames */
#define	FGRAB_CAP_MASK		0x000000f0
#define	FGRAB_NTSC		0x00000100
#define	FGRAB_PAL		0x00000200
#define	FGRAB_SECAM		0x00000400
#define	FGRAB_AUTOMODE		0x00000800
#define	FGRAB_FORM_MASK		0x00000f00

#define	FGRAB_DEV0		0x00001000
#define	FGRAB_DEV1		0x00002000
#define	FGRAB_DEV2		0x00003000
#define	FGRAB_DEV3		0x00004000
#define FGRAB_DEV_SVIDEO	0x00005000
#define FGRAB_DEV_RGB		0x00006000
#define	FGRAB_DEV_MASK		0x00007000
#define FGRAB_EXT_SYNC          0x00008000

#define FGRAB_MONO8		0x00000000
#define	FGRAB_RGB16		0x00010000
#define	FGRAB_RGB24		0x00020000
#define	FGRAB_YUV_PACKED	0x00030000
#define	FGRAB_YUV_9		0x00040000
#define	FGRAB_YUV_12		0x00050000
#define	FGRAB_YUV_16		0x00060000
#define	FGRAB_OUTPUT_FMT_MASK	0x000f0000

#define	FGRAB_WANT_EVEN		0x00100000
#define	FGRAB_WANT_ODD		0x00200000
#define	FGRAB_WANT_MASK		0x00300000

#define FGRAB_INTERLACE_FIELDS	0x00000000
#define FGRAB_ONLY_EVEN_FIELDS	0x01000000
#define FGRAB_ONLY_ODD_FIELDS	0x02000000
#define FGRAB_ALTERNATE_FIELDS	0x03000000
#define FGRAB_NON_INTERLACE_FIELDS 0x04000000
#define FGRAB_FIELDS_MASK  	0x07000000

#define	FGRAB_WANT_TS		0x08000000	  /* time-stamp a frame */

#define FGRAB_EXPECT_FIELD	0x20000000	/* expected field on next interrupt     */
#define FGRAB_ENABLE_INTR       0x40000000      /* flag that interrupts are enabled     */

#define FGRAB_STATUS_EVEN	0x01
#define FGRAB_STATUS_ODD	0x02
#define FGRAB_STATUS_FIFO_ERR	0x04
#define FGRAB_STATUS_DMA_ERR	0x08
#define FGRAB_STATUS_START      0x10
#define FGRAB_STATUS_BUS_ERR    0x20

#define FGRAB_INTR_PROC_ACTIVE	0x01
#define FGRAB_INTR_NEED_LOOP	0x02

    
#define MAX_NUM_FRAMES		256 /* or what YOU think is reasonable 	*/
struct fgrab {
   kdev_t minor;		/* minor device number for this board	*/
   void *frame_buffer;		/* buffer that holds the captured image */
   u_int fb_size;		/* Size of buffer 			*/
   struct task_struct *proc;	/* process to receive raised signal 	*/
   int pid;			/* PID of said process 			*/
   struct wait_queue *waitq;	/* Queue of sleeping procs 		*/
   int		signal;		/* signal to send to process 		*/
   int		sigmode;	/* when to signal 			*/
   struct fgrab_mem *mem;	/* used to control sync. multi-frame output */
   u_long	synch_wait;	/* wait for free buffer before continuing   */
   short	cur_frame;	/* frame number in buffer (1-frames) 	*/
   short	next_frame;	/* frame number of next frame in buffer */
   short	rows;		/* number of rows in a frame 		*/
   short	cols;		/* number of columns in a frame 	*/
   short	depth;		/* number of bits per pixel 		*/
   short	frames;		/* number of frames allocated 		*/
   int		frame_size;	/* number of bytes in a frame 		*/

   u_long	fifo_errors;	/* number of fifo capture errors since open */
   u_long	dma_errors;	/* number of DMA capture errors since open  */
   u_long	bus_errors;	/* number of PCI bus     errors since open  */

   u_long	frames_captured;/* number of frames captured since open */
   u_long	even_fields_captured; /* number of even fields captured */
   u_long	odd_fields_captured;  /* number of odd fields captured  */
   unsigned	flags;
   u_short	fps;		/* frames per second */
   u_long       dma_add_e[3];
   u_long       dma_add_o[3];
   u_long       dma_str_e[3];
   u_long       dma_str_o[3];
   u_long	frame_offset[MAX_NUM_FRAMES];
   /* Functions called by generic frame grabber code			*/
   /* This is a pointer to a structure which in turn contains pointers
    * to routines.  This is done so that init only requires passing the
    * pointer for each board rather than copying the list for each board
    */
   struct fgrab_progs *progs;
   /* Pointer to data that is private to the above subroutines		*/
   void *data;
   int irq;	/* irq assigned to this device				*/
 };

struct fgrab_progs	{
    void (*open)(unsigned minor);
    void (*close)(unsigned minor);
    void (*unwind)(unsigned minor);
    int  (*set_dma_e)(unsigned minor,
                      unsigned long *off_e, unsigned long *stride_e,
                      unsigned long base, unsigned long fb_size) ;
    int  (*set_dma_o)(unsigned minor,
                      unsigned long *off_o, unsigned long *stride_o,
                      unsigned long base, unsigned long fb_size) ;
    void (*enable_dma)(unsigned minor);
    void (*disable_dma)(unsigned minor);
    void (*start_capture)(unsigned minor,unsigned single_flag);
    void (*stop_capture)(unsigned minor);
    int (*check_intr)(unsigned minor);
    void (*enable_intr)(unsigned minor);
    void (*disable_intr)(unsigned minor);
    int (*check_err)(unsigned minor);
    void (*clear_err)(unsigned minor);
    int (*set_fps)(unsigned minor, u_short val);
    int (*status)(unsigned minor);
#define SETPROG(name) void (*set_##name)(unsigned minor, u_long arg_val);    
#define GETCPROG(name) unsigned (*get_##name)(unsigned minor);    
    SETPROG(hue)
    GETCPROG(hue)
    SETPROG(chroma_gain)
    GETCPROG(chroma_gain)
    SETPROG(brightness)
    GETCPROG(brightness)
    SETPROG(chroma_saturation)
    GETCPROG(chroma_saturation)
    SETPROG(contrast)
    GETCPROG(contrast)
    SETPROG(hor_wndw_strt)
    GETCPROG(hor_wndw_strt)
    SETPROG(vert_wndw_strt)
    GETCPROG(vert_wndw_strt)
    SETPROG(pixel_clock)
    GETCPROG(pixel_clock)
#define SETPROGNA(name) int (*set_##name)(unsigned minor);    
    SETPROGNA(input_dev_0)
    SETPROGNA(input_dev_1)
    SETPROGNA(input_dev_2)
    SETPROGNA(input_dev_3)
    SETPROGNA(input_dev_svideo)
    SETPROGNA(input_dev_rgb)
    SETPROG(input_ext_sync)
    SETPROGNA(fmt_ntsc)
    SETPROGNA(fmt_pal)
    SETPROGNA(fmt_secam)
    SETPROGNA(fmt_auto)
    int (*set_field_odd_only)(unsigned minor);
    int (*set_field_even_only)(unsigned minor);
    int (*set_field_interlace)(unsigned minor);
    int (*set_field_alternate)(unsigned minor);
    int (*set_field_non_interlace)(unsigned minor);
    int (*set_frame)(unsigned minor, int rows, int cols, int frames);
#define SETGEO(name) int (*set_geo_##name)(unsigned minor);    
    SETGEO(rgb16)
    SETGEO(rgb24)
    SETGEO(mono8)
    SETGEO(yuv_9)
    SETGEO(yuv_12)
    SETGEO(yuv_16)
    SETGEO(yuv_packed)
#undef SETGEO
    void (*set_chroma_signed)(unsigned minor, unsigned flag);    
#undef SETPROG
#undef SETPROGNA
#undef GETCPROG
    int (*ioctl)(unsigned minor, unsigned cmd, u_long *arg_val, int *write_flag);
    int (*ckioctl)(unsigned minor, unsigned ioctl, unsigned function);
    } ;

#ifndef   FGRAB_INTERNAL
extern 
int  fgrab_register(int irq,                    /* desired irq              */
                    struct fgrab_progs *progs,  /* board level routines     */
                    unsigned int d_size,        /* size of data area        */
                    void **d_ptr,               /* place to return pointer  */
                    unsigned int *dev) ;        /* minor device             */
extern
void fgrab_unregister(unsigned minor) ;
extern
void fgrab_wait_jiffies(u_long waitJiffies) ;
#endif /* FGRAB_INTERNAL */

#define FGRAB_WAIT_MS2JIF(t_ms) ((HZ*(t_ms)+999)/1000+1) 


#ifdef DEBUG_FGRAB
#define CAT(a)\
printk(a"\n")

#define SHOW(a)\
printk(#a": %d 0x%x\n", a, a);

#define SHOW2(a,b)\
printk(#a": %d 0x%x "#b": %d 0x%x\n", a, a, b, b);

#define PRINTK(a) printk a

#else

#define PRINTK(a) /* */

#endif

/*
 * Following is from drivers/scsi/hosts.c of 2.0.18.
 * Overrides for Emacs so that we follow Linus's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-indent-level: 4
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -4
 * c-argdecl-indent: 4
 * c-label-offset: -4
 * c-continued-statement-offset: 4
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */
