/*    meteor.c  -- Meteor board specific code for Generic Frame Grabber.
 *    
 *    See file COPYING for copyright info
 *
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/mm.h>

#include "fgrab.h"
#include "fgrab_pci.h"
#include "saa7116.h"
#include "saa7196.h"
#include "bt254.h"
#include "meteor.h"
#include "meteor_ioctl.h"


/*
 * meteor_detect is the only externally known symbol
 * It must be included in the module symbol table.
 */
int meteor_detect(void) ;
static struct symbol_table meteor_syms = {
#include <linux/symtab_begin.h>
    X(meteor_detect),
#include <linux/symtab_end.h>
} ;    


#if defined(PAL)
#define METEOR_MAXFPS 25
#elif defined(SECAM)
#define METEOR_MAXFPS 25
#else
#define METEOR_MAXFPS 30
#endif

static struct meteor *meteor[MAXFGRABS];

static struct fgrab_progs meteor_progs ;

static int meteor_register(unsigned int irq,struct saa7116 *saa7116) ;
static int meteor_signature(void) ;
static void unwind(unsigned minor) ;
static void meteor_init(unsigned minor) ;
static void open(unsigned minor) ;
static void close(unsigned minor) ;
static void stop_capture(unsigned minor) ;
static int set_fps(unsigned minor, u_short fps) ;
static void saa7116_init(unsigned minor) ;
static void saa7116_reset(unsigned minor) ;
static int set_input_dev_0(unsigned minor) ;
static int set_input_dev_1(unsigned minor) ;
static int set_input_dev_2(unsigned minor) ;
static int set_input_dev_3(unsigned minor) ;
static int set_input_dev_svideo(unsigned minor) ;
static int  set_input_dev_rgb(unsigned minor) ;
static int set_fmt_ntsc(unsigned minor) ;
static int set_fmt_pal(unsigned minor) ;
static int set_fmt_secam(unsigned minor) ;
static int set_fmt_auto(unsigned minor) ;
static int set_field_odd_only(unsigned minor) ;
static int set_field_even_only(unsigned minor) ;
static int set_field_interlace(unsigned minor) ;
static int set_field_alternate(unsigned minor) ;
static int set_frame(unsigned minor, int rows, int cols, int frames) ;
static int set_geo_rgb16(unsigned minor) ;
static int set_geo_rgb24(unsigned minor) ;
static int set_geo_mono8(unsigned minor) ;
static int set_geo_yuv_9(unsigned minor) ;
static int set_geo_yuv_12(unsigned minor) ;  
static int set_geo_yuv_16(unsigned minor) ;
static int set_geo_yuv_packed(unsigned minor) ;
static int ioctl(unsigned minor, unsigned cmd, u_long *arg_val,int *write_flag) ;
static int ckioctl(unsigned minor, unsigned ioctl, unsigned function) ;



static void setup_progs(void) {
    meteor_progs.open                   = open ;
    meteor_progs.close                  = close ;
    meteor_progs.unwind                 = unwind ;
    meteor_progs.set_dma_e              = saa7116_progs.set_dma_e ;
    meteor_progs.set_dma_o              = saa7116_progs.set_dma_o ;
    meteor_progs.enable_dma             = fgrab_pci_progs.enable_dma ;
    meteor_progs.disable_dma            = fgrab_pci_progs.disable_dma ;
    meteor_progs.start_capture          = saa7116_progs.start_capture ;
    meteor_progs.stop_capture           = stop_capture ;
    meteor_progs.check_intr             = saa7116_progs.check_intr ;
    meteor_progs.enable_intr            = saa7116_progs.enable_intr ;
    meteor_progs.disable_intr           = saa7116_progs.disable_intr ;
    meteor_progs.check_err              = saa7116_progs.check_err ;
    meteor_progs.clear_err              = saa7116_progs.clear_err ;
    meteor_progs.set_fps                = set_fps ;
    meteor_progs.set_hue                = saa7196_progs.set_hue ;
    meteor_progs.get_hue                = saa7196_progs.get_hue ;
    meteor_progs.set_chroma_gain        = saa7196_progs.set_chroma_gain ;
    meteor_progs.get_chroma_gain        = saa7196_progs.get_chroma_gain ;
    meteor_progs.set_brightness         = saa7196_progs.set_brightness ;
    meteor_progs.get_brightness         = saa7196_progs.get_brightness ;
    meteor_progs.set_chroma_saturation  = saa7196_progs.set_chroma_saturation ;
    meteor_progs.get_chroma_saturation  = saa7196_progs.get_chroma_saturation ;
    meteor_progs.set_contrast           = saa7196_progs.set_contrast ;
    meteor_progs.get_contrast           = saa7196_progs.get_contrast  ;
    meteor_progs.set_hor_wndw_strt      = saa7196_progs.set_hor_wndw_strt ;
    meteor_progs.get_hor_wndw_strt      = saa7196_progs.get_hor_wndw_strt ;
    meteor_progs.set_vert_wndw_strt     = saa7196_progs.set_vert_wndw_strt ;
    meteor_progs.get_vert_wndw_strt     = saa7196_progs.get_vert_wndw_strt ;
    meteor_progs.set_pixel_clock        = NULL ;
    meteor_progs.get_pixel_clock        = NULL ;
    meteor_progs.set_input_dev_0        = set_input_dev_0 ;
    meteor_progs.set_input_dev_1        = set_input_dev_1 ;
    meteor_progs.set_input_dev_2        = set_input_dev_2 ;
    meteor_progs.set_input_dev_3        = set_input_dev_3 ;
    meteor_progs.set_input_dev_svideo   = set_input_dev_svideo ;
    meteor_progs.set_input_dev_rgb      = set_input_dev_rgb ;
    meteor_progs.set_input_ext_sync     = NULL ;
    meteor_progs.set_fmt_ntsc           = set_fmt_ntsc ;
    meteor_progs.set_fmt_pal            = set_fmt_pal ;
    meteor_progs.set_fmt_secam          = set_fmt_secam ;
    meteor_progs.set_fmt_auto           = set_fmt_auto ;
    meteor_progs.set_field_odd_only     = set_field_odd_only ;
    meteor_progs.set_field_even_only    = set_field_even_only ;
    meteor_progs.set_field_interlace    = set_field_interlace ;
    meteor_progs.set_field_alternate    = set_field_alternate ;
    meteor_progs.set_field_non_interlace= set_field_alternate ;
    meteor_progs.set_frame              = set_frame ;
    meteor_progs.set_geo_rgb16          = set_geo_rgb16 ;
    meteor_progs.set_geo_rgb24          = set_geo_rgb24 ;
    meteor_progs.set_geo_mono8          = NULL ;        /* 7116 with 7196 doesn't have mono-8 mode. RWF */
#ifdef METEOR_NOPLANAR
    meteor_progs.set_geo_yuv_9          = NULL ;
    meteor_progs.set_geo_yuv_12         = NULL ;
    meteor_progs.set_geo_yuv_16         = NULL ;
#else
    meteor_progs.set_geo_yuv_9          = set_geo_yuv_9 ;
    meteor_progs.set_geo_yuv_12         = set_geo_yuv_12 ;     
    meteor_progs.set_geo_yuv_16         = set_geo_yuv_16 ;
#endif
    meteor_progs.set_geo_yuv_packed     = set_geo_yuv_packed ;
    meteor_progs.set_chroma_signed      = saa7196_progs.set_chroma_signed ;
    meteor_progs.ioctl                  = ioctl ;      
    meteor_progs.ckioctl                = ckioctl ;      
}


/* Designed to be used as a module */
#ifdef MODULE
void cleanup_module(void) {
    unsigned minor;
    for (minor=0;minor<MAXFGRABS;minor++) {
        if (meteor[minor] != NULL) {
            unwind(minor);
        }
    }
}
int init_module(void) {
    register_symtab(&meteor_syms);
    return(meteor_detect());
}
#endif /*MODULE*/

static void meteor_setup(void) {
    int i;
    for (i=0;i<MAXFGRABS;i++) meteor[i] = NULL ;
    setup_progs();
}

/*
 * Detect presence of boards.
 * Initialize the boards, data structures, etc.
 */
int meteor_detect(void) {
    int err;
    meteor_setup();
    PRINTK((KERN_INFO "meteor: ready to call saa7116 detect routine\n"));
    err=saa7116_progs.detect(meteor_register,meteor_signature);
    return err;
}

/* signature determines whether a particular 7116 board is a
   meteor board or some other 7116 based board                              */
static int meteor_signature(void) {
    /*
     * I don't know what a valid meteor signature is RWF 30JAN97
     * Could kludge this by verifying that it isn't a dt315x board
     * used to do something even more degenerate - just say its a meteor
     * if we get an error when we try to talk to the dt i2c id register 
     * then assume we have a meteor. RWF 18JUN97
     */
    saa7116_init(FGRAB_MINOR_SETUP);
    return( (saa7116_progs.talk_i2c(FGRAB_MINOR_SETUP,METEOR_DT315X_ID_R,0,0)==0) ? 0 : 1);
}

static int meteor_register(unsigned int irq,struct saa7116 *saa7116) {
    int err;
    struct meteor *mtr;
    unsigned int minor;
    if ((err=fgrab_register(irq,
                            &meteor_progs,
                            sizeof(struct meteor),
                            (void *)&mtr,&minor))) {
        printk(KERN_ERR "fgrab: meteor: cannot register for irq %d\n", irq);
        return(err);
    }
    meteor[minor] = mtr ;
    mtr->minor = minor ;
    mtr->irq = irq ;
    mtr->saa7116 = *saa7116;
    saa7116_progs.setup(&(mtr->saa7116),irq,minor);
    saa7196_progs.setup(&(mtr->saa7196),irq,minor);
    bt254_progs.setup(&(mtr->bt254),irq,minor);
    meteor_init(minor);
    printk(KERN_INFO "Meteor driver version " METEOR_VERSION 
           ": minor %d ready\n", minor);
    return(0);
}

/*
 * Initialize the 7116, 7196 and the RGB module.
 */
static void
meteor_init(unsigned minor)
{
  /*
   * Initialize the Philips SAA7116
   */
    saa7116_init(minor);
    saa7116_progs.probe_range(minor);
  /*
   * Initialize the Philips SAA7196
   */
  saa7196_progs.init(minor);
  /*
   * Initialize RGB module
   */
  meteor[minor]->flags |= (bt254_progs.init(minor)) ? METEOR_RGB : 0 ;
  set_fps(minor, METEOR_MAXFPS);
}

/* open is the operations that need to be done when a process 
   claims the device                                                    */
/* reinitialize device registers                                        */
	/*
	 * Make sure that the i2c regs are set the same for each open.
	 */
static void
open(unsigned minor) {
    MOD_INC_USE_COUNT;
    saa7116_init(minor);
    saa7196_progs.reset(minor);
    set_fps(minor, METEOR_MAXFPS);
}

static void
close(unsigned minor)
{
    saa7116_reset(minor);
    MOD_DEC_USE_COUNT;
}

/*
 * These routines with name prefixes of saa7116_ are at the board level
 * because they have delays that seem to vary from board to board.
 */

/*
 * Initialize the saa7116.
 * The incantations for this seem to differ from board to board RWF
 */

static void saa7116_init(unsigned minor) {
    saa7116_reset(minor);
    /* capt_ctl = 0x000080c0  Bring out of reset, enable range check */
    saa7116_progs.set_capt_ctl(minor, 
                              SAA7116_CSR1_RANGE_EN | SAA7116_CSR1_FIFO_EN | SAA7116_CSR1_SRST );
    fgrab_wait_jiffies(METEOR_WAIT_JIFFS);
    saa7116_progs.set_i2c_clocks(minor, 0x46, 0x1e, 0x1e, 0x0f) ;       /* 0x461e1e0f */
    saa7116_progs.set_geo_rgb16(minor);
    saa7116_progs.set_fifo_trigger(minor, SAA7116_PLANAR | SAA7116_PACKED ) ;
    saa7116_progs.enable_field_toggle(minor) ;
    saa7116_progs.reset_int_flags(minor) ;      /* Clear and disable all interrupts */
    /* This combination of masks and mask length means capture all fields */
    saa7116_progs.set_field_masks(minor, 0x00000001, 0, 0x00000001, 0) ;
    saa7116_progs.set_fifo_limits(minor) ;
    fgrab_wait_jiffies(METEOR_WAIT_JIFFS);	/* Let this sink in */
}

/*
 * Reset the saa7116.
 */
static void saa7116_reset(unsigned minor) {
    fgrab_wait_jiffies(METEOR_WAIT_JIFFS);		/* Let things settle down      */
    saa7116_progs.set_capt_ctl(minor, 0x00000000) ;     /* First, put into reset       */
    saa7116_progs.zero_regs(minor) ;                    /* Clear and reset everything  */
    fgrab_wait_jiffies(METEOR_WAIT_JIFFS);		/* Let the dust settle         */
}

static void stop_capture(unsigned minor) {
    saa7116_progs.stop_capture(minor);
}

static int 
set_fps(unsigned minor, u_short fps)
{
    int maxfps, mask, length;
    int b, step;

    if (fps<1)
	return (-1);

    maxfps=saa7196_progs.get_max_fps(minor) ;
    if (maxfps==0) {    /* if no signal use default     */
        maxfps = METEOR_MAXFPS ;
    }
    if ( (fps > maxfps) || (fps < 1) ) return(-1) ;
    if (fps == maxfps) {
        mask = 0x1;
        length = 0x0;
    } else if (2*fps == maxfps) {
        mask = 0x1;
        length = 0x1;
    } else if (fps > maxfps/2) {
        int den = maxfps - fps ;
        mask = (1<<maxfps) - 1;
        length = maxfps - 1;
        step = (maxfps - 1) ;
        for (b=step; b<maxfps*den; b+=step) {
            mask &= ~(1<<(b/den));  /* mask out the bth frame */
        }
    } else {
        mask = 0x1;
        length = maxfps - 1;
        step = (maxfps - 1) ;
        for (b=step+fps; b<(maxfps-1)*fps; b+=step) {
            mask |= (1<<(b/fps));  /* mask in the bth frame */
        }
    }

    saa7116_progs.set_field_masks(minor,mask,length,mask,length);
    return 0;
}



static void unwind(unsigned minor) {
    saa7116_reset(minor);
    saa7116_progs.unwind(minor) ;
    fgrab_unregister(minor);
}

#define SETPROG(name) static int set_##name(unsigned minor)
#define GETPROG(name) static unsigned int get_##name(unsigned minor) 
#define SETPROGNA(name) static int set_##name(unsigned minor)

SETPROGNA(input_dev_0) { 
    struct meteor *mtr = meteor[minor] ;
    if(mtr->flags & METEOR_RGB)
        bt254_progs.select_saa7196(minor);
    saa7196_progs.set_gpsw(minor,0x0);
    saa7196_progs.set_chrominance_trap(minor,1);
    return(0);
}

SETPROGNA(input_dev_1) { 
    struct meteor *mtr = meteor[minor] ;
    if(mtr->flags & METEOR_RGB)
        bt254_progs.select_saa7196(minor);
    saa7196_progs.set_gpsw(minor,0x1);
    saa7196_progs.set_chrominance_trap(minor,1);
    return(0);
}

SETPROGNA(input_dev_2) { 
    struct meteor *mtr = meteor[minor] ;
    if(mtr->flags & METEOR_RGB)
        bt254_progs.select_saa7196(minor);
    saa7196_progs.set_gpsw(minor,0x2);
    saa7196_progs.set_chrominance_trap(minor,1);
    return(0);
}

SETPROGNA(input_dev_3) { 
    struct meteor *mtr = meteor[minor] ;
    if(mtr->flags & METEOR_RGB)
        bt254_progs.select_saa7196(minor);
    saa7196_progs.set_gpsw(minor,0x3);
    saa7196_progs.set_chrominance_trap(minor,1);
    return(0);
}

SETPROGNA(input_dev_svideo) { 
    struct meteor *mtr = meteor[minor] ;
    if(mtr->flags & METEOR_RGB)
        bt254_progs.select_saa7196(minor);
    saa7196_progs.set_gpsw(minor,0x2);
    saa7196_progs.set_chrominance_trap(minor,0);
    return(0);
}


SETPROGNA(input_dev_rgb) {
    if(meteor[minor]->flags & METEOR_RGB)
        return(-EINVAL);
    saa7196_progs.set_gpsw(minor,0x3);
    saa7196_progs.set_chrominance_trap(minor,1);
    bt254_progs.select_bt254(minor);
    saa7196_progs.set_gpsw(minor,0x3);
    return(0);
}

SETPROGNA(fmt_ntsc) { 
    struct meteor *mtr = meteor[minor] ;
    mtr->flags = (mtr->flags&~METEOR_FORM_MASK) | METEOR_NTSC ;
    saa7196_progs.stdc_secam_mode(minor,0);
    saa7196_progs.ctrl1_lum_delay(minor,0x40);
    saa7196_progs.inpix_per_line(minor,0x280);
    saa7196_progs.lines_per_field(minor,0x0f0);
    if (mtr->flags & METEOR_RGB) {
        bt254_progs.ntsc(minor, 1);
    }
    return(0);
}

SETPROGNA(fmt_pal) { 
    struct meteor *mtr = meteor[minor] ;
    mtr->flags = (mtr->flags&~METEOR_FORM_MASK) | METEOR_PAL ;
    saa7196_progs.stdc_secam_mode(minor,0);
    saa7196_progs.ctrl1_lum_delay(minor,0x0);
    saa7196_progs.inpix_per_line(minor,0x300);
    saa7196_progs.lines_per_field(minor,0x120);
    if(mtr->flags & METEOR_RGB){
        bt254_progs.ntsc(minor, 0);			  
    }
    return(0);
}

SETPROGNA(fmt_secam) { 
    struct meteor *mtr = meteor[minor] ;
    mtr->flags = (mtr->flags&~METEOR_FORM_MASK) | METEOR_SECAM ;
    saa7196_progs.stdc_secam_mode(minor,1);
    saa7196_progs.ctrl1_lum_delay(minor,0x20);
    saa7196_progs.inpix_per_line(minor,0x300);
    saa7196_progs.lines_per_field(minor,0x120);
    if(mtr->flags & METEOR_RGB){
        bt254_progs.ntsc(minor, 0);			  
    }
    return(0);
}

SETPROGNA(fmt_auto) { 
    struct meteor *mtr = meteor[minor] ;
    mtr->flags = (mtr->flags&~METEOR_FORM_MASK) | METEOR_AUTOMODE ;
    saa7196_progs.stdc_secam_mode(minor,0);
    saa7196_progs.ctrl1_lum_delay(minor,0x80);
    return(0);
}


static int set_field_odd_only(unsigned minor) {
    struct meteor *mtr = meteor[minor] ;
    mtr->flags = (mtr->flags & ~METEOR_FIELDS_MASK) 
        | METEOR_ONLY_ODD_FIELDS ;
    saa7196_progs.set_field_odd_only(minor);
    saa7116_progs.set_field_odd_only(minor);
    return(0);
}
static int set_field_even_only(unsigned minor) {
    struct meteor *mtr = meteor[minor] ;
    mtr->flags = (mtr->flags & ~METEOR_FIELDS_MASK) 
        | METEOR_ONLY_EVEN_FIELDS ;
    saa7196_progs.set_field_even_only(minor);
    saa7116_progs.set_field_even_only(minor);
    return(0);
}
static int set_field_alternate(unsigned minor) {
    struct meteor *mtr = meteor[minor] ;
    mtr->flags = (mtr->flags & ~METEOR_FIELDS_MASK) 
        | METEOR_ALTERNATE_FIELDS ;
    saa7196_progs.set_field_alternate(minor);
    saa7116_progs.set_field_both(minor);
    return(0);
}
static int set_field_interlace(unsigned minor) {
    struct meteor *mtr = meteor[minor] ;
    mtr->flags = (mtr->flags & ~METEOR_FIELDS_MASK) 
        | METEOR_INTERLACE_FIELDS ;
    saa7196_progs.set_field_interlace(minor);
    saa7116_progs.set_field_both(minor);
    return(0);
}
    
static int set_frame(unsigned minor, int rows, int cols, int frames) {
    struct meteor *mtr = meteor[minor] ;
    int error=0;

    /* The dma address increments in 4 byte steps, which means dma_end will
     * only be correct if columns%4 == 0.  For YUV we require columns%8 == 0
     * since for each 8 bits of Y there are 4 bits each for U and V.  This bit
     * of code takes the conservative option.  In field mode mode rows don't
     * need to be even, but we're conservative here again.  In field mode the 
     * number of fields specified (geo->frames) must be even since the board 
     * will only stop capture after odd fields.
     */
    if ((cols & 0x3f8) != cols) {
        PRINTK((KERN_INFO
                "fgrab%ld: meteor: %d: columns too large or not diviside by 8.\n",
                mtr->minor, cols));
        error = -EINVAL;
    }
    if (((rows & 0x7fe) != rows) ||
        (( (mtr->flags & METEOR_FIELDS_MASK) == METEOR_ALTERNATE_FIELDS ) &&
         ((rows & 0x3fe) != rows)) ) {
	PRINTK((KERN_INFO
		"fgrab%ld: meteor: %d: rows too large or not even.\n",
		mtr->minor, rows));
	error = -EINVAL;
    }
    if (((mtr->flags & METEOR_FIELDS_MASK)==METEOR_ALTERNATE_FIELDS) && 
        (frames > 1) && 
        (frames & 0x1) == 0x1) {
	PRINTK((KERN_INFO
		"fgrab%ld: meteor: %d: must have an even number of fields in field mode.\n",
		mtr->minor, frames));
	error = -EINVAL;
      }
    if (error!=0) return error;

    mtr->rows = rows ;
    mtr->cols = cols ;
    /* set cols */
    saa7196_progs.set_cols(minor,cols);
    /* set rows */
    switch(mtr->flags&METEOR_FIELDS_MASK) {
    case METEOR_ONLY_ODD_FIELDS:
    case METEOR_ONLY_EVEN_FIELDS:
    case METEOR_ALTERNATE_FIELDS:
        saa7196_progs.set_rows(minor,rows);
        break;
    case METEOR_INTERLACE_FIELDS:
        saa7196_progs.set_rows(minor,rows>>1);
        break;
    }
    /*
     * This is here to allow control of image size without
     * resampling. It overrides values set when the image
     * format (eg ntsc etc) was selected. RWF 18JUN97
     */
    if (mtr->flags&METEOR_NO_RESAMP) {
        saa7196_progs.lines_per_field(minor,rows>>1);
        saa7196_progs.inpix_per_line(minor,cols);
    }
    return(0);
}


SETPROG(geo_rgb16) { 
    saa7196_progs.set_oformat_rgb16(minor);
    saa7116_progs.set_geo_rgb16(minor);
    return(0);
}

SETPROG(geo_rgb24) { 
    saa7196_progs.set_oformat_rgb24(minor);
    saa7116_progs.set_geo_rgb24(minor);
    return(0);
}

/* 
 * These are guesses RWF 21JUN97
 * These don't work -- apparently 7116 with 7196 doesn't
 * give a clean mono mode.  Best that can be done is
 * 2 duplicate bytes for each pixel -- not really any
 * different than you16 throwing away uv. RWF 29JUN97
 */
SETPROG(geo_mono8) { 
    saa7196_progs.set_oformat_mono(minor);
    saa7116_progs.set_geo_rgb16(minor);
    return(0);
}

SETPROG(geo_yuv_9) { 
    saa7116_progs.set_geo_yuv_9(minor);
    saa7196_progs.set_oformat_yuv(minor);
    return(0);
}

SETPROG(geo_yuv_12) {
    saa7116_progs.set_geo_yuv_12(minor);
    saa7196_progs.set_oformat_yuv(minor);
    return(0);
}

SETPROG(geo_yuv_16) {
    saa7116_progs.set_geo_yuv_16(minor);
    saa7196_progs.set_oformat_yuv(minor);
    return(0);
}

SETPROG(geo_yuv_packed) {
    saa7196_progs.set_oformat_yuv(minor);
    saa7116_progs.set_geo_yuv_packed(minor);
    return(0);
}

static int 
ioctl(unsigned minor, unsigned cmd, u_long *arg_val,int *write_flag) {
    struct meteor *mtr = meteor[minor] ;
    int	error = 0 ;  

    switch (cmd) {

    case METEORSBT254:
        if((mtr->flags & METEOR_RGB) == 0)
            return -EINVAL;
        bt254_progs.write(minor, *arg_val & 0xf, (*arg_val & 0x0ff0) >> 4);
        *write_flag = 0 ;
        break;
    case METEORGBT254:
        if((mtr->flags & METEOR_RGB) == 0)
            return -EINVAL;
        *arg_val = bt254_progs.get_reg(minor, *arg_val);
        *write_flag = 1 ;
        break;

    case METEORGCAPT:	/* get capt_ctl */
    {
        *arg_val = saa7116_progs.get_capt_ctl(minor) ;
        *write_flag = 1 ;
        break;
    }

    case METEORRESAMP:
        mtr->flags &= ~METEOR_NO_RESAMP ;
        switch(mtr->flags&METEOR_FORM_MASK) {
        case METEOR_NTSC:
            saa7196_progs.inpix_per_line(minor,0x280);
            saa7196_progs.lines_per_field(minor,0x0f0);
            break;
        case METEOR_PAL:
        case METEOR_SECAM:
            saa7196_progs.inpix_per_line(minor,0x300);
            saa7196_progs.lines_per_field(minor,0x120);
            break;
        case METEOR_AUTOMODE:
            /* Not sure what appropriate action here for automode is RWF 23JUN97    */
            break;
        }
        break;
    case METEORNORESAMP:
        mtr->flags |= METEOR_NO_RESAMP ;
        saa7196_progs.lines_per_field(minor,mtr->rows>>1);
        saa7196_progs.inpix_per_line(minor,mtr->cols);
        break;

    case METEORSTATUS:
        *arg_val = saa7196_progs.status(minor);
        *write_flag = 1 ;
        break;

    default:
        PRINTK((KERN_INFO "fgrab%d: ioctl: invalid ioctl request 0x%x\n", minor, cmd));
        error = -ENOTTY;
        break;
    }  
    return(error);

}

static int ckioctl(unsigned minor, unsigned ioctl, unsigned function) {
    struct meteor *mtr = meteor[minor] ;
    int error = 0 ;
    switch(ioctl) {
    case METEORSBT254:
        if((mtr->flags & METEOR_RGB) == 0) error = -EINVAL ;
        break;
    case METEORGBT254:
        if((mtr->flags & METEOR_RGB) == 0) error = -EINVAL ;
        break;
    case METEORGCAPT:	/* get capt_ctl */
    case METEORRESAMP:
    case METEORNORESAMP:
    case METEORSTATUS:
        break;
    default:
        error = -ENOTTY;
        break;
    }  
    return(error);
}


/*
 * Following is from drivers/scsi/hosts.c of 2.0.18.
 * Overrides for Emacs so that we follow Linus's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-indent-level: 4
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -4
 * c-argdecl-indent: 4
 * c-label-offset: -4
 * c-continued-statement-offset: 4
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */
