/*
 *	saa71x6.h  -- Definition of the saa7116 registers
 *    
 *    See file COPYING for copyright info
 *
 */

/*
 * The registers of the saa7116 video chip.
 */

/*
 * Bitfield words are each defined as a separate struct. This unclutters
 * the actual register struct definitions, and also enables use of gcc
 * constructor expressions for assignment, which is readable and results
 * in better object code than individual assignments to bitfields.
 *
 * Note bitfields build from LSB up.
 *
 * Slight variances from actual field width are made to use byte boundaries
 * when possible. Actual widths are noted in comments.
 */

/* Bitfields of the 7116 */

struct rt_mode {
  unsigned mode:8;		/* 0:7  Pixel format for even fields */
  unsigned route:24;		/* 8:31 Pixel router mode for even fields */
};

struct ff_trig {
  unsigned packed:7;		/* 0:7(0:6) Fifo trigger Packed Mode */
  unsigned :9;			/* 8:15(7:15) Unused */
  unsigned planar:7;		/* 16:31(16:22) Fifo trigger Planar Mode */
  unsigned :9;			/* 23:31 Unused */
};

struct i2c_stat {
  unsigned :9;			/* 0:8 Not needed */
  unsigned d_abort:1;		/* 9:9 Status of i2c write: 0==success */
  unsigned :14;			/* 10:23 Not needed */
  unsigned r_data:8;		/* 24:31 Data returned by i2c read */
};

struct i2c_cmd {
  unsigned w_data:8;		/* 0:7 Data to be written out */
  unsigned reg_addr:8;		/* 8:15 Register address in device space */
  unsigned dev_addr:8;		/* 16:23 16: 0==write; 17:23: Device address */
  unsigned xfer:1;		/* 24: Set to start write. Clears when done */
  unsigned :7;			/* 25:31 Unused */
};
#define v_struct volatile struct


				/* (pointer-addition-values)hex-offset(s) */
struct saa7116_reg {
  u_long dma_e[3];		/* (0-2)00-08: Base address, even fields */
  u_long dma_o[3];		/* (3-5)0c-14: Base address, odd fields */
  u_long stride_e[3];		/* (6-8)18-20: Address stride, even fields */
  u_long stride_o[3];		/* (9-11)24-2c: Address stride, odd fields */
  struct rt_mode rt_mode_e;	/* (12)30: Route and Mode for even fields */
  struct rt_mode rt_mode_o;	/* (13)34: Route and Mode for odd fields */
  struct ff_trig fifo_trigger;	/* (14)38 Fifo triggers: Planar and Packed */
  u_long field_toggle;		/* (15)3c */
  volatile u_long capt_ctl;	/* (16)40: Capture control, etc. (CSR1) */
  u_long retry_wait;		/* (17)44 */
  volatile u_long ints_ctl;	/* (18)48: Interrupt control and status (INT_CSR) */
  u_long field_mask_e;		/* (19)4c */
  u_long field_mask_o;		/* (20)50 */
  u_long mask_lengths;		/* (21)54 */
  u_long fifo_limits;		/* (22)58 */
  u_long i2c_clocks;		/* (23)5c */
  v_struct i2c_stat i2c_stat;	/* (24)60 i2c mode control and status */
  v_struct i2c_cmd i2c_cmd;	/* (25)64 Direct mode command and data */
  u_long i2c_auto_0_e;		/* (26)68 */
  u_long i2c_auto_2_e;		/* (27)6c */
  u_long i2c_auto_4_e;		/* (28)70 */
  u_long i2c_auto_6_e;		/* (29)74 */
  u_long i2c_auto_0_o;		/* (30)78 */
  u_long i2c_auto_2_o;		/* (31)7c */
  u_long i2c_auto_4_o;		/* (32)80 */
  u_long i2c_auto_6_o;		/* (33)84 */
  u_long i2c_regs_enable;	/* (34)88 */
  u_long dma_end_e;		/* (35)8c */
  u_long dma_end_o;		/* (36)90 */
};

#undef v_struct

struct saa7116 {
    struct saa7116_reg *reg;
    unsigned minor;
    struct fgrab_pci pci;
    u_long status ;
    u_long cap_err;
    u_long bus_err;
    u_long flags;
} ;


struct saa7116_progs {
    int (*detect)(int (*dev_register)(unsigned irq,struct saa7116 *saa7116),
                  int (*dev_signature)(void)) ;
    void (*unwind)(unsigned minor) ;
    void (*setup)(struct saa7116 *saa7116,int irq, unsigned minor) ;
    int  (*set_dma_e)(unsigned minor,
                      unsigned long *off_e, unsigned long *stride_e,
                      unsigned long base, unsigned long fb_size) ;
    int  (*set_dma_o)(unsigned minor,
                      unsigned long *off_o, unsigned long *stride_o,
                      unsigned long base, unsigned long fb_size) ;
    void (*start_capture)(unsigned minor, unsigned single_flag) ;
    void (*stop_capture)(unsigned minor) ;
    int  (*check_intr)(unsigned minor) ;
    void (*enable_intr)(unsigned minor) ;
    void (*disable_intr)(unsigned minor) ;
    int  (*check_err)(unsigned minor) ;
    void (*clear_err)(unsigned minor) ;
    void (*zero_regs)(unsigned minor) ;
    void (*set_capt_ctl)(unsigned minor, unsigned val) ;
    void (*set_i2c_clocks)(unsigned minor, 
                           unsigned phase_4, unsigned phase_3, 
                           unsigned phase_2, unsigned phase_1) ;
    void (*set_fifo_trigger)(unsigned minor, unsigned flag) ;
    void (*enable_field_toggle)(unsigned minor) ;
    void (*disable_field_toggle)(unsigned minor) ;
    void (*reset_int_flags)(unsigned minor) ;
    void (*set_fifo_limits)(unsigned minor) ;
    void (*probe_range)(unsigned minor) ;
    void (*set_field_masks)(unsigned minor, 
                            unsigned field_mask_e,unsigned length_e,
                            unsigned field_mask_o,unsigned length_o) ;
    void (*set_field_odd_only)(unsigned minor) ;
    void (*set_field_even_only)(unsigned minor) ;
    void (*set_field_both)(unsigned minor) ;
    void (*set_geo_rgb16)(unsigned minor) ;
    void (*set_geo_rgb24)(unsigned minor) ;
    void (*set_geo_yuv_9)(unsigned minor) ;
    void (*set_geo_yuv_12)(unsigned minor) ;
    void (*set_geo_yuv_16)(unsigned minor) ;
    void (*set_geo_yuv_packed)(unsigned minor) ;
    unsigned (*get_capt_ctl)(unsigned minor) ;
    int (*get_i2c_data)(unsigned minor) ;
    int (*talk_i2c)(unsigned minor, u_char dev, u_char addr, u_char data) ;
    } ;


#ifndef   SAA7116_INTERNAL
extern struct saa7116_progs saa7116_progs ;
#endif /* SAA7116_INTERNAL */

#define SAA7116_PACKED 0x01
#define SAA7116_PLANAR 0x02

/*
 * The following definitions are for the register at offset 0x40
 * DT and Philips documentation refer to this as CSR1
 * The meteor driver referred to this as capt_ctl
 */
#define SAA7116_CSR1_RANGE_EN           ((0x1)<<15)     /* Range check enable            0x8000 */
#define SAA7116_CSR1_CRPT_DIS           ((0x1)<<14)     /* Corrupt disable                      */
/* skip 12-13 */
#define SAA7116_CSR1_ADDR_ERR_ODD       ((0x1)<<11)     /* Address error odd                    */
#define SAA7116_CSR1_ADDR_ERR_EVE       ((0x1)<<10)     /* Address error even                   */
#define SAA7116_CSR1_FLD_CRPT_ODD       ((0x1)<< 9)     /* Field corrupt odd                    */
#define SAA7116_CSR1_FLD_CRPT_EVE       ((0x1)<< 8)     /* Field corrupt even                   */
#define SAA7116_CSR1_FIFO_EN            ((0x1)<< 7)     /* FIFO enable                   0x0080 */
#define SAA7116_CSR1_SRST               ((0x1)<< 6)     /* Software controlled reset (low true) */
#define SAA7116_CSR1_FLD_DN_ODD         ((0x1)<< 5)     /* Field done odd                       */
#define SAA7116_CSR1_FLD_DN_EVE         ((0x1)<< 4)     /* Field done even                      */
#define SAA7116_CSR1_CAP_SNGl_ODD       ((0x1)<< 3)     /* Capture single field odd             */
#define SAA7116_CSR1_CAP_SNGL_EVE       ((0x1)<< 2)     /* Capture single field even            */
#define SAA7116_CSR1_CAP_CONT_ODD       ((0x1)<< 1)     /* Capture continuous odd fields        */
#define SAA7116_CSR1_CAP_CONT_EVE       ((0x1)<< 0)     /* Capture continuous even fields       */
/* Derived masks */
#define SAA7116_CSR1_DMA_ERR_MASK                       \
              ( SAA7116_CSR1_ADDR_ERR_ODD |             \
                SAA7116_CSR1_ADDR_ERR_EVE )
#define SAA7116_CSR1_FIFO_ERR_MASK                      \
              ( SAA7116_CSR1_FLD_CRPT_ODD |             \
                SAA7116_CSR1_FLD_CRPT_EVE )
#define SAA7116_CSR1_ERR_MASK                           \
              ( SAA7116_CSR1_DMA_ERR_MASK  |            \
                SAA7116_CSR1_FIFO_ERR_MASK )
#define SAA7116_CSR1_DN_MASK                            \
              ( SAA7116_CSR1_FLD_DN_ODD   |             \
                SAA7116_CSR1_FLD_DN_EVE   )

/*
 * The following definitions are for the register at offset 0x48
 * DT and Philips documentation refer to this as INT_CSR
 * The meteor driver referred to this as ints_ctl
 */
#define SAA7116_FLD_START_EN            ((0x1)<<10)     /* Interrupt mask, start of field       */
#define SAA7116_FLD_END_ODD_EN          ((0x1)<< 9)     /* Interrupt mask, end of odd field     */
#define SAA7116_FLD_END_EVE_EN          ((0x1)<< 8)     /* Interrupt mask, end of even field    */
#define SAA7116_FLD_START               ((0x1)<< 2)     /* Interrupt status, start of field     */
#define SAA7116_FLD_END_ODD             ((0x1)<< 1)     /* Interrupt status, end of odd field   */
#define SAA7116_FLD_END_EVE             ((0x1)<< 0)     /* Interrupt status, end of even field  */
/* Derived masks */
#define SAA7116_INTR_EN_MASK ( SAA7116_FLD_START_EN     |       \
                               SAA7116_FLD_END_ODD_EN   |       \
                               SAA7116_FLD_END_EVE_EN )
#define SAA7116_INTR_ST_MASK ( SAA7116_FLD_START        |       \
                               SAA7116_FLD_END_ODD      |       \
                               SAA7116_FLD_END_EVE    )


#define SAA7116_IIC_READ(reg)  (((reg)<<1)|0x1)
#define SAA7116_IIC_WRITE(reg) (((reg)<<1)|0x0)
#define _IIC_R(reg) SAA7116_IIC_READ(reg)
#define _IIC_W(reg) SAA7116_IIC_WRITE(reg)

#define SAA7116_ONLY_EVEN_FIELDS	0x01000000
#define SAA7116_ONLY_ODD_FIELDS	        0x02000000
#define SAA7116_FIELDS_MASK  	        0x03000000

#define SAA7116_WAIT_JIFFS ((HZ*10+999)/1000+1) /* wait at least 10 msec                        */
#define SAA7116_FIFO_TRIGGER            0x40    /* fifo level at which DMA starts (dwords)      */
#define SAA7116_FIFO_ALMOST_EMPTY       0x20    /* fifo level where DMA shutdown begins (dwords)*/
#define SAA7116_FIFO_ALMOST_FULL        0x60    /* fifo level where overflow declared (dwords)  */
/*
 * Phillips doucmentation suggests trigger of 0x20, almost_empty of 0x05, and almost_full of 0x7c.
 * Note that the SAA7116 documentation from Phillips says that ALMOST_FULL of 0x7c is a good
 * value to utilize the whole FIFO.  But 0x80 dwords corresponds to 512 bytes.  The FIFO is
 * supposed to be 1024 bytes long so this would seem to leave half of the FIFO untouched.  
 * I'm not sure what piece of this puzzle I am missing.  R. Fries 29jul97
 */


/*
 * Following is from drivers/scsi/hosts.c of 2.0.18.
 * Overrides for Emacs so that we follow Linus's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-indent-level: 4
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -4
 * c-argdecl-indent: 4
 * c-label-offset: -4
 * c-continued-statement-offset: 4
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */
