/*
 *      saa7196 chip specific routines
 *    
 *    See file COPYING for copyright info
 *
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/mm.h>
/*---------------------------------------------------------
**
**	saa7196 routines
**
**---------------------------------------------------------
*/

#define SAA7196_INTERNAL
#include "saa7196.h"
#undef SAA7196_INTERNAL
#include "fgrab_pci.h"
#include "saa7116.h"
#include "fgrab.h"

/* declarations of routines that will appear in saa7196_progs           */
static void setup(struct saa7196 *saa7196,int irq, unsigned minor);
static void init(unsigned minor) ;
static void reset(unsigned minor) ;
static int get_max_fps(unsigned minor) ;
static void set_chroma_signed(unsigned minor, unsigned flag) ;
static void set_gpsw(unsigned minor, unsigned val) ;
static void set_chrominance_trap(unsigned minor, unsigned active_flag) ;
static void stdc_secam_mode(unsigned minor, unsigned select);
static void ctrl1_lum_delay(unsigned minor,unsigned val) ;
static void inpix_per_line(unsigned minor,unsigned val) ;
static void lines_per_field(unsigned minor,unsigned val) ;
static void set_field_odd_only(unsigned minor) ;
static void set_field_even_only(unsigned minor) ;
static void set_field_alternate(unsigned minor) ;
static void set_field_interlace(unsigned minor) ;
static void set_cols(unsigned minor,unsigned cols) ;
static void set_rows(unsigned minor, unsigned rows) ;
static void set_oformat_rgb16(unsigned minor) ;
static void set_oformat_yuv(unsigned minor) ;
static void set_oformat_rgb24(unsigned minor) ;
static void set_oformat_mono(unsigned minor) ;
static int status(unsigned minor) ;
static void set_hue(unsigned minor, u_long arg_val) ;
static unsigned int get_hue(unsigned minor) ;
static void set_chroma_gain(unsigned minor, u_long arg_val) ;
static unsigned int get_chroma_gain(unsigned minor) ;
static void set_brightness(unsigned minor, u_long arg_val) ;
static unsigned int get_brightness(unsigned minor) ;
static void set_chroma_saturation(unsigned minor, u_long arg_val) ;
static unsigned int get_chroma_saturation(unsigned minor) ;
static void set_contrast(unsigned minor, u_long arg_val) ;
static unsigned int get_contrast(unsigned minor) ;
static void set_hor_wndw_strt(unsigned minor, u_long arg_val) ;
static unsigned int get_hor_wndw_strt(unsigned minor) ;
static void set_vert_wndw_strt(unsigned minor, u_long arg_val) ;
static unsigned int get_vert_wndw_strt(unsigned minor) ;


struct saa7196_progs saa7196_progs = {
    setup, init, reset,
    get_max_fps, set_chroma_signed, set_gpsw,
    set_chrominance_trap, stdc_secam_mode, ctrl1_lum_delay, inpix_per_line,
    lines_per_field, 
    set_field_odd_only, set_field_even_only, 
    set_field_alternate, set_field_interlace,
    set_cols, set_rows,
    set_oformat_rgb16, set_oformat_yuv, set_oformat_rgb24, set_oformat_mono,
    status,
    set_hue, get_hue,
    set_chroma_gain, get_chroma_gain,
    set_brightness, get_brightness,
    set_chroma_saturation, get_chroma_saturation,
    set_contrast, get_contrast,
    set_hor_wndw_strt, get_hor_wndw_strt,
    set_vert_wndw_strt, get_vert_wndw_strt
} ;

static struct symbol_table saa7196_syms = {
#include <linux/symtab_begin.h>
    X(saa7196_progs),
#include <linux/symtab_end.h>
} ;    



struct saa7196 *saa7196[MAXFGRABS];

static u_char saa7196_i2c_default[NUM_SAA7196_I2C_REGS] = {
			/* SAA7196 I2C bus control			*/
			/* BITS	Function				*/
/* 00 */	0x50,	/* 7:0	Increment Delay				*/
/* 01 */	0x30,	/* 7:0	Horizontal Sync Begin for 50hz		*/
/* 02 */	0x00,	/* 7:0	Horizontal Sync Stop for 50hz		*/
/* 03 */	0xe8,	/* 7:0	Horizontal Sync Clamp Start for 50hz	*/
/* 04 */	0xb6,	/* 7:0	Horizontal Sync Clamp Stop for 50hz 	*/
/* 05 */	0xf4,	/* 7:0	Horizontal Sync Start after PH1 for 50hz */
/* 06 */	0x46,	/*   7	Input mode =0 CVBS, =1 S-Video 
			     6	Pre filter
			   5:4  Aperture Bandpass characteristics
			   3:2	Coring range for high freq
			   1:0	Aperture bandpass filter weights	*/
/* 07 */	0x00,	/* 7:0	Hue					*/
/* 08 */	0x7f,	/* 7:3	Colour-killer threshold QAM (PAL, NTSC) */
/* 09 */	0x7f,	/* 7:3	Colour-killer threshold SECAM		*/
/* 0a */	0x7f,	/* 7:0	PAL switch sensitivity			*/
/* 0b */	0x7f,	/* 7:0	SECAM switch sensitivity		*/
/* 0c */	0x40,	/*   7	Colour-on bit
			   6:5	AGC filter				*/
/* 0d */	0x84,	/*   7	VTR/TV mode bit = 1->VTR mode
			     3	Realtime output mode select bit
			     2	HREF position select
			     1	Status byte select
			     0	SECAM mode bit				*/
/* 0e */	0x38,	/*   7	Horizontal clock PLL
			     5	Select interal/external clock source
			     4	Output enable of Horizontal/Vertical sync
			     3	Data output YUV enable
			     2	S-VHS bit
			     1	GPSW2
			     0	GPSW1					*/
/* 0f */	0x50,	/*   7	Automatic Field detection
			     6	Field Select 0 = 50hz, 1=60hz
			     5	SECAM cross-colour reduction
			     4	Enable sync and clamping pulse
			   3:1	Luminance delay compensation		*/
/* 10 */	0x00,	/*   2	Select HREF Position
			   1:0  Vertical noise reduction		*/
/* 11 */	0x2c,	/* 7:0	Chrominance gain conrtol for QAM	*/
/* 12 */	0x40,	/* 7:0	Chrominance saturation control for VRAM port */
/* 13 */	0x40,	/* 7:0	Luminance contract control for VRAM port */
/* 14 */	0x34,	/* 7:0	Horizontal sync begin for 60hz		*/
#ifdef notdef
/* 15 */	0x0c,	/* 7:0	Horizontal sync stop for 60hz		*/
/* 16 */	0xfb,	/* 7:0	Horizontal clamp begin for 60hz		*/
/* 17 */	0xd4,	/* 7:0	Horizontal clamp stop for 60hz		*/
/* 18 */	0xec,	/* 7:0	Horizontal sync start after PH1 for 60hz */
#else
		0x0a, 0xf4, 0xce, 0xf4,
#endif
/* 19 */	0x80,	/* 7:0	Luminance brightness control for VRAM port */
/* 1a */	0x00,
/* 1b */	0x00,
/* 1c */	0x00,
/* 1d */	0x00,
/* 1e */	0x00,
/* 1f */	0x00,
/* 20 */	0x90,	/*   7	ROM table bypass switch
			   6:5	Set output field mode
			     4	VRAM port outputs enable
			   3:2	First pixel position in VRO data
			   1:0	FIFO output register select		*/
/* 21 */	0x80,	/* 7:0	[7:0] Pixel number per line on output	*/
/* 22 */	0x80,	/* 7:0	[7:0] Pixel number per line on input	*/
/* 23 */	0x03,	/* 7:0	[7:0] Horizontal start position of scaling win*/
/* 24 */	0x8a,	/* 7:5	Horizontal decimation filter
			     4  [8] Horizontal start position of scaling win
			   3:2	[9:8] Pixel number per line on input
			   1:0  [9:8] Pixel number per line on output 	*/
/* 25 */	0xf0,	/* 7:0	[7:0] Line number per output field	*/
/* 26 */	0xf0,	/* 7:0	[7:0] Line number per input field	*/
/* 27 */	0x0f,	/* 7:0	[7:0] Vertical start of scaling window	*/
/* 28 */	0x80,	/*   7	Adaptive filter switch
			   6:5	Vertical luminance data processing
			     4	[8] Vertical start of scaling window 
			   3:2  [9:8] Line number per input field
			   1:0	[9:8] Line number per output field	*/
/* 29 */	0x16,	/* 7:0	[7:0] Vertical bypass start		*/
/* 2a */	0x00,	/* 7:0	[7:0] Vertical bypass count		*/
/* 2b */	0x00,	/*   4  [8] Vertical bypass start
			     2  [8] Vertical bypass count
			     0	Polarity, internally detected odd even flag */
/* 2c */	0x80,	/* 7:0	Set lower limit V for colour-keying	*/
/* 2d */	0x7f,	/* 7:0	Set upper limit V for colour-keying	*/
/* 2e */	0x80,	/* 7:0	Set lower limit U for colour-keying	*/
/* 2f */	0x7f,	/* 7:0	Set upper limit U for colour-keying	*/
/* 30 */	0xbf	/*   7	VRAM bus output format
			     6	Adaptive geometrical filter
			     5	Luminance limiting value
			     4	Monochrome and two's complement output data sel
			     3	Line quailifier flag
			     2	Pixel qualifier flag
			     1	Transparent data transfer
			     0	Extended formats enable bit		*/
};


static void setup(struct saa7196 *saa7196ptr,int irq, unsigned minor) {
    saa7196[minor] = saa7196ptr;
}

/* Designed to be used as a module */
#ifdef MODULE

void cleanup_module(void) {
}


int init_module(void) {
    register_symtab(&saa7196_syms);
    return(0);
}

#endif /*MODULE*/

static int 
get_max_fps(unsigned minor) {
    unsigned status;
    int fps;
    SAA7196_WRITE(minor, SAA7196_STDC, 
                  SAA7196_REG(minor, SAA7196_STDC) | 0x02);
    SAA7196_READ(minor);
    status = (saa7116_progs.get_i2c_data(minor)) ;
    if((status & 0x40) == 0) {
	/* got video signal */
	if (status & 0x20) {
	    fps = 30;
	} else {
	    /* 50Hz system */
	    fps = 25;
	}
    } else {
        fps = 0 ;
    }
    return(fps);
}


/*
 * Initialize the 7196
 */
static void
init(unsigned minor)
{
  /*
   * Check for the Philips SAA7196
   */
  if(saa7116_progs.talk_i2c(minor, SAA7196_I2C_ADDR_W, 0, 0xff) == 0) {
    /*
     * Initialize 7196
     */
    reset(minor);
#ifdef DEBUG_FGRAB
    /*
     * Get version number.
     */
    SAA7196_WRITE(minor, SAA7196_STDC,
		  SAA7196_REG(minor, SAA7196_STDC) & ~0x02);
    SAA7196_READ(minor);
    printk(KERN_INFO "fgrab%d: <Philips SAA 7196> rev 0x%x\n",
	   minor, (saa7116_progs.get_i2c_data(minor)>>4));
#endif DEBUG_FGRAB

  }
}

static void
reset(unsigned minor) {
    int i;
    saa7196[minor]->fmts = saa7196_i2c_default[SAA7196_FMTS] ;
    for(i=0; i< NUM_SAA7196_I2C_REGS; i++) {
        SAA7196_WRITE(minor, i, saa7196_i2c_default[i]);
    }
}

static int status(unsigned minor) {
    u_short stat;
    SAA7196_WRITE(minor, SAA7196_STDC,
                  SAA7196_REG(minor, SAA7196_STDC) | 0x02);
    SAA7196_READ(minor);
    stat = saa7116_progs.get_i2c_data(minor) ;
    SAA7196_WRITE(minor, SAA7196_STDC,
                  SAA7196_REG(minor, SAA7196_STDC) & ~0x02);
    SAA7196_READ(minor);
    stat |= (saa7116_progs.get_i2c_data(minor) << 8);
    return stat;
}


#define SETPROG(name) static void set_##name(unsigned minor, u_long arg_val)
#define GETPROG(name) static unsigned int get_##name(unsigned minor) 

SETPROG(hue) { 
    SAA7196_WRITE(minor, SAA7196_HUEC, (char)arg_val);
}
GETPROG(hue) { 
    return (SAA7196_REG(minor, SAA7196_HUEC));
}

SETPROG(chroma_gain) { 
    SAA7196_WRITE(minor, SAA7196_CGAINR, (char)arg_val);
}
GETPROG(chroma_gain) { 
    return (SAA7196_REG(minor, SAA7196_CGAINR));
}

SETPROG(brightness) { 
    SAA7196_WRITE(minor, SAA7196_BRIG, (char)arg_val);
}
GETPROG(brightness) { 
    return (SAA7196_REG(minor, SAA7196_BRIG));
}

SETPROG(chroma_saturation) { 
    SAA7196_WRITE(minor, SAA7196_CSAT, (char)arg_val);
}
GETPROG(chroma_saturation) { 
    return(SAA7196_REG(minor, SAA7196_CSAT));
}

SETPROG(contrast) { 
    SAA7196_WRITE(minor, SAA7196_CONT, (char)arg_val);
}
GETPROG(contrast) { 
    return(SAA7196_REG(minor, SAA7196_CONT));
}



SETPROG(hor_wndw_strt) { 
    SAA7196_WRITE(minor, SAA7196_HWS, arg_val);
}
GETPROG(hor_wndw_strt) { 
    return(SAA7196_REG(minor, SAA7196_HWS));
}

SETPROG(vert_wndw_strt) { 
    SAA7196_WRITE(minor, SAA7196_VWS, arg_val);
}
GETPROG(vert_wndw_strt) { 
    return(SAA7196_REG(minor, SAA7196_VWS));
}

static void set_chroma_signed(unsigned minor, unsigned flag) {
     SAA7196_WRITE(minor, SAA7196_DPATH, 
                   (SAA7196_REG(minor, SAA7196_DPATH) & ~SAA7196_DPATH_MCT) |
                   ((!flag)?0:SAA7196_DPATH_MCT) ) ;
}

static void
set_gpsw(unsigned minor, unsigned val) {
    SAA7196_WRITE(minor, SAA7196_IOCC,
                  (SAA7196_REG(minor, SAA7196_IOCC) & ~0x3) | (val&0x3));
}

static void
set_chrominance_trap(unsigned minor,unsigned active_flag) {
    SAA7196_WRITE(minor, SAA7196_LUMC,
                  (SAA7196_REG(minor, SAA7196_LUMC) & ~0x80)
                  | ((active_flag) ? 0 : 0x80)      );
}

static void
stdc_sel_status_byte(unsigned minor,unsigned select) {
    SAA7196_WRITE(minor, SAA7196_STDC, 
                  (SAA7196_REG(minor, SAA7196_STDC) & ~0x02)
                  | ( (select) ? 0x02 : 0 )  );
}

static void
stdc_secam_mode(unsigned minor,unsigned select) {
    SAA7196_WRITE(minor, SAA7196_STDC, 
                  (SAA7196_REG(minor, SAA7196_STDC) & ~0x01)
                  | ( (select) ? 0x01 : 0 )  );
}

static void
ctrl1_lum_delay(unsigned minor,unsigned val) {
    /* note the 7196 spec says luminance delay is bits 0-2 but the meteor
       driver seems to think is is 1-3  -- this follows the driver RWF  */
    SAA7196_WRITE(minor, SAA7196_CTRL1, 
                  (SAA7196_REG(minor, SAA7196_CTRL1) & ~0x0e)
                  | ( val & 0xe )  );
}

static void
inpix_per_line(unsigned minor,unsigned val) {
    SAA7196_WRITE(minor, SAA7196_INPIX, val & 0xff);
    SAA7196_WRITE(minor, SAA7196_HFILT, 
                  (SAA7196_REG(minor, SAA7196_HFILT) & ~0x0c)
                  | ( 0xc & (val>>(8-2)) ) );
}

static void
lines_per_field(unsigned minor,unsigned val) {
    SAA7196_WRITE(minor, SAA7196_INLINE, val & 0xff);
    SAA7196_WRITE(minor, SAA7196_VYP, 
                  (SAA7196_REG(minor, SAA7196_VYP) & ~0x0c)
                  | ( 0xc & (val>>(8-2)) ) );
}

static void
set_cols(unsigned minor,unsigned cols) {
    SAA7196_WRITE(minor, SAA7196_OUTPIX, cols & 0xff);
    SAA7196_WRITE(minor, SAA7196_HFILT,
                  ((SAA7196_REG(minor, SAA7196_HFILT) & ~0x03) |
                   ((cols >> 8) & 0x03)));
}

static void
set_rows(unsigned minor, unsigned rows) {
	SAA7196_WRITE(minor, SAA7196_OUTLINE, ((rows) & 0xff));
	SAA7196_WRITE(minor, SAA7196_VYP,
		      ((SAA7196_REG(minor, SAA7196_VYP) & ~0x03) |
		       (((rows) >> 8) & 0x03)));
}

static void
set_field_odd_only(unsigned minor) {
    saa7196[minor]->fmts = ((saa7196[minor]->fmts) & ~SAA7196_FMTS_FIELD_MASK)
                           | SAA7196_FMTS_ODD_ONLY 
                           | SAA7196_FMTS_RTB | SAA7196_FMTS_VPE ;
    SAA7196_WRITE(minor, SAA7196_FMTS, saa7196[minor]->fmts);
}
static void
set_field_even_only(unsigned minor) {
    saa7196[minor]->fmts = ((saa7196[minor]->fmts) & ~SAA7196_FMTS_FIELD_MASK)
                           | SAA7196_FMTS_EVEN_ONLY 
                           | SAA7196_FMTS_RTB | SAA7196_FMTS_VPE ;
    SAA7196_WRITE(minor, SAA7196_FMTS, saa7196[minor]->fmts);
}
static void
set_field_alternate(unsigned minor) {
    saa7196[minor]->fmts = ((saa7196[minor]->fmts) & ~SAA7196_FMTS_FIELD_MASK)
                           | SAA7196_FMTS_ALTERNATE
                           | SAA7196_FMTS_RTB | SAA7196_FMTS_VPE ;
    SAA7196_WRITE(minor, SAA7196_FMTS, saa7196[minor]->fmts);
}
static void
set_oformat_rgb16(unsigned minor) {
    saa7196[minor]->fmts = ((saa7196[minor]->fmts) & ~SAA7196_FMTS_OFMT_MASK)
                           | SAA7196_FMTS_RGB16 ;
    SAA7196_WRITE(minor, SAA7196_FMTS, saa7196[minor]->fmts);
}
static void
set_oformat_yuv(unsigned minor) {
    saa7196[minor]->fmts = ((saa7196[minor]->fmts) & ~SAA7196_FMTS_OFMT_MASK)
                           | SAA7196_FMTS_YUV ;
    SAA7196_WRITE(minor, SAA7196_FMTS, saa7196[minor]->fmts);
}
static void
set_oformat_rgb24(unsigned minor) {
    saa7196[minor]->fmts = ((saa7196[minor]->fmts) & ~SAA7196_FMTS_OFMT_MASK)
                           | SAA7196_FMTS_RGB24 ;
    SAA7196_WRITE(minor, SAA7196_FMTS, saa7196[minor]->fmts);
}
/*
 * This doesn't work -- I guess it results in the wrong signals making
 * it to the 7116.  Running mono with TTR=1 gives duplicate bytes which
 * in effect buys nothing over yuv16 and throwing away uv info. 
 * Guess 7116 with 7196 doesn't have clean mono mode. RWF 29JUN97
 */
static void
set_oformat_mono(unsigned minor) {
    saa7196[minor]->fmts = ((saa7196[minor]->fmts) & ~SAA7196_FMTS_OFMT_MASK)
                           | SAA7196_FMTS_MONO ;
    SAA7196_WRITE(minor, SAA7196_FMTS, saa7196[minor]->fmts);
    SAA7196_WRITE(minor, SAA7196_DPATH, 
                  SAA7196_REG(minor,SAA7196_DPATH)&~(SAA7196_DPATH_TTR|SAA7196_DPATH_MCT) );
}

static void
set_field_interlace(unsigned minor) {
    saa7196[minor]->fmts = ((saa7196[minor]->fmts) & ~SAA7196_FMTS_FIELD_MASK)
                           | SAA7196_FMTS_INTERLACE
                           | SAA7196_FMTS_RTB | SAA7196_FMTS_VPE ;
    SAA7196_WRITE(minor, SAA7196_FMTS, saa7196[minor]->fmts);
}



/*
 * Following is from drivers/scsi/hosts.c of 2.0.18.
 * Overrides for Emacs so that we follow Linus's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-indent-level: 4
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -4
 * c-argdecl-indent: 4
 * c-label-offset: -4
 * c-continued-statement-offset: 4
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */
