/*
 * Definitions for the Philips SAA7196 digital video decoder,
 * scalar, and clock generator circuit (DESCpro).
 *    
 *    See file COPYING for copyright info
 *
 */

#define NUM_SAA7196_I2C_REGS	49
#define	SAA7196_I2C_ADDR_W	0x40
#define	SAA7196_I2C_ADDR_R	0x41

#define SAA7196_WRITE(minor, reg, data)                                 \
    saa7116_progs.talk_i2c(minor, SAA7196_I2C_ADDR_W, reg, data),       \
    saa7196[minor]->i2c[reg] = data; /* Update the saved image */
#define	SAA7196_READ(minor)                                             \
    saa7116_progs.talk_i2c(minor, SAA7196_I2C_ADDR_R, 0x0, 0x0)

#define SAA7196_REG(minor, reg) saa7196[minor]->i2c[reg]


#define SAA7196_IDEL	0x00	/* Increment delay */
#define SAA7196_HSB5	0x01	/* H-sync begin; 50 hz */
#define SAA7196_HSS5	0x02	/* H-sync stop; 50 hz */
#define SAA7196_HCB5	0x03	/* H-clamp begin; 50 hz */
#define SAA7196_HCS5	0x04	/* H-clamp stop; 50 hz */
#define SAA7196_HSP5	0x05	/* H-sync after PHI1; 50 hz */
#define SAA7196_LUMC	0x06	/* Luminance control */
#define SAA7196_HUEC	0x07	/* Hue control */
#define SAA7196_CKTQ	0x08	/* Colour Killer Threshold QAM (PAL, NTSC) */
#define SAA7196_CKTS	0x09	/* Colour Killer Threshold SECAM */
#define SAA7196_PALS	0x0a	/* PAL switch sensitivity */
#define SAA7196_SECAMS	0x0b	/* SECAM switch sensitivity */
#define SAA7196_CGAINC	0x0c	/* Chroma gain control */
#define SAA7196_STDC	0x0d	/* Standard/Mode control */
#define SAA7196_IOCC	0x0e	/* I/O and Clock Control */
#define SAA7196_CTRL1	0x0f	/* Control #1 */
#define SAA7196_CTRL2	0x10	/* Control #2 */
#define SAA7196_CGAINR	0x11	/* Chroma Gain Reference */
#define SAA7196_CSAT	0x12	/* Chroma Saturation */
#define SAA7196_CONT	0x13	/* Luminance Contrast */
#define SAA7196_HSB6	0x14	/* H-sync begin; 60 hz */
#define SAA7196_HSS6	0x15	/* H-sync stop; 60 hz */
#define SAA7196_HCB6	0x16	/* H-clamp begin; 60 hz */
#define SAA7196_HCS6	0x17	/* H-clamp stop; 60 hz */
#define SAA7196_HSP6	0x18	/* H-sync after PHI1; 60 hz */
#define SAA7196_BRIG	0x19	/* Luminance Brightness */
#define SAA7196_FMTS	0x20	/* Formats and sequence */
#define SAA7196_OUTPIX	0x21	/* Output data pixel/line */
#define SAA7196_INPIX	0x22	/* Input data pixel/line */
#define SAA7196_HWS	0x23	/* Horiz. window start */
#define SAA7196_HFILT	0x24	/* Horiz. filter */
#define SAA7196_OUTLINE	0x25	/* Output data lines/field */
#define SAA7196_INLINE	0x26	/* Input data lines/field */
#define SAA7196_VWS	0x27	/* Vertical window start */
#define SAA7196_VYP	0x28	/* AFS/vertical Y processing */
#define SAA7196_VBS	0x29	/* Vertical Bypass start */
#define SAA7196_VBCNT	0x2a	/* Vertical Bypass count */
#define SAA7196_VBP	0x2b	/* veritcal Bypass Polarity */
#define SAA7196_VLOW	0x2c	/* Colour-keying lower V limit */
#define SAA7196_VHIGH	0x2d	/* Colour-keying upper V limit */
#define SAA7196_ULOW	0x2e	/* Colour-keying lower U limit */
#define SAA7196_UHIGH	0x2f	/* Colour-keying upper U limit */
#define SAA7196_DPATH	0x30	/* Data path setting  */

#define SAA7196_FMTS_FIELD_MASK (0x3<<5)
#define SAA7196_FMTS_INTERLACE  (0x0<<5)
#define SAA7196_FMTS_ALTERNATE  (0x1<<5)
#define SAA7196_FMTS_ODD_ONLY   (0x2<<5)
#define SAA7196_FMTS_EVEN_ONLY  (0x3<<5)
#define SAA7196_FMTS_RTB        (0x80)
#define SAA7196_FMTS_VPE        (0x10)
#define SAA7196_FMTS_OFMT_MASK  (0x3<<0)
#define SAA7196_FMTS_RGB16      (0x0<<0)
#define SAA7196_FMTS_YUV        (0x1<<0)
#define SAA7196_FMTS_RGB24      (0x2<<0)
#define SAA7196_FMTS_MONO       (0x3<<0)

/*
 * DPATH register bit definitions.  Default setting is 0xbf
 * VOF |    | LLV | MCT | QPL | QPP | TTR | EFE (i.e. only AFG is left out).
 * mono is special case where also leavr out TTR and MCT
 */
#define SAA7196_DPATH_VOF       (0x1<<7)        /* VRAM buf output format (1 disables 16 to 32 multiplex)       */
#define SAA7196_DPATH_AFG       (0x1<<6)        /* adaptive geometrical filter (1 enables)                      */
#define SAA7196_DPATH_LLV       (0x1<<5)        /* luminance limiting value 0=>1-254, 1=>16-235 (mono and yuv)  */
#define SAA7196_DPATH_MCT       (0x1<<4)        /* monochrome and twos complement output data select            */
#define SAA7196_DPATH_QPL       (0x1<<3)        /* line qualifier polarilty flag 0=>active low  1=> active high */
#define SAA7196_DPATH_QPP       (0x1<<2)        /* pixel qualifier polarity flag 0=>active low  1=> active high */
#define SAA7196_DPATH_TTR       (0x1<<1)        /* transparent data transfer 0=>VRAM burst 1=>FIFO transparent  */
#define SAA7196_DPATH_EFE       (0x1<<0)        /* extended formats enable bit 0=>32 bit output 1=>extended     */



struct saa7196 {
    u_char	i2c[NUM_SAA7196_I2C_REGS];/* saa7196 register values */
    unsigned fmts;
} ;

struct saa7196_progs {
    void (*setup)(struct saa7196 *saa7196,int irq, unsigned minor);
    void (*init)(unsigned minor) ;
    void (*reset)(unsigned minor) ;
    int (*get_max_fps)(unsigned minor) ;
    void (*set_chroma_signed)(unsigned minor, unsigned flag) ;
    void (*set_gpsw)(unsigned minor, unsigned val) ;
    void (*set_chrominance_trap)(unsigned minor, unsigned active_flag) ;
    void (*stdc_secam_mode)(unsigned minor, unsigned select);
    void (*ctrl1_lum_delay)(unsigned minor,unsigned val) ;
    void (*inpix_per_line)(unsigned minor,unsigned val) ;
    void (*lines_per_field)(unsigned minor,unsigned val) ;
    void (*set_field_odd_only)(unsigned minor) ;
    void (*set_field_even_only)(unsigned minor) ;
    void (*set_field_alternate)(unsigned minor) ;
    void (*set_field_interlace)(unsigned minor) ;
    void (*set_cols)(unsigned minor,unsigned cols) ;
    void (*set_rows)(unsigned minor, unsigned rows) ;
    void (*set_oformat_rgb16)(unsigned minor) ;
    void (*set_oformat_yuv)(unsigned minor) ;
    void (*set_oformat_rgb24)(unsigned minor) ;
    void (*set_oformat_mono)(unsigned minor) ;
    int (*status)(unsigned minor) ;
    void (*set_hue)(unsigned minor, u_long arg_val) ;
    unsigned int (*get_hue)(unsigned minor) ;
    void (*set_chroma_gain)(unsigned minor, u_long arg_val) ;
    unsigned int (*get_chroma_gain)(unsigned minor) ;
    void (*set_brightness)(unsigned minor, u_long arg_val) ;
    unsigned int (*get_brightness)(unsigned minor) ;
    void (*set_chroma_saturation)(unsigned minor, u_long arg_val) ;
    unsigned int (*get_chroma_saturation)(unsigned minor) ;
    void (*set_contrast)(unsigned minor, u_long arg_val) ;
    unsigned int (*get_contrast)(unsigned minor) ;
    void (*set_hor_wndw_strt)(unsigned minor, u_long arg_val) ;
    unsigned int (*get_hor_wndw_strt)(unsigned minor) ;
    void (*set_vert_wndw_strt)(unsigned minor, u_long arg_val) ;
    unsigned int (*get_vert_wndw_strt)(unsigned minor) ;
} ;


#ifndef   SAA7196_INTERNAL
extern struct saa7196_progs saa7196_progs ;
#endif /* SAA7196_INTERNAL */

/*
 * Following is from drivers/scsi/hosts.c of 2.0.18.
 * Overrides for Emacs so that we follow Linus's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-indent-level: 4
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -4
 * c-argdecl-indent: 4
 * c-label-offset: -4
 * c-continued-statement-offset: 4
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */
