/*    fgrab_ioctl.h  -- Ioctl generic definitions for Generic Frame Grabber.
 *    
 *    See file COPYING for copyright info
 *
 */

/*
 *	ioctl constants for Fgrab Capture Device.
 */

#ifndef _MACHINE_IOCTL_FGRAB_H
#define _MACHINE_IOCTL_FGRAB_H

#define MAX_NUM_FRAMES 256 /* should be the same as fgrab.h */

#include <sys/ioctl.h>
struct fgrab_capframe {
	short	command;	/* see below for valid FGRABCAPFRM commands */
	short	lowat;		/* start transfer if < this number */
	short	hiwat;		/* stop transfer if > this number */
} ;

/* structure for FGRAB[GS]ETGEO - get/set geometry  */
struct fgrab_geomet {
	u_short		rows;
	u_short		columns;
	u_short		frames;
	u_long		oformat;
} ;

/* structure for FGRABGCOUNT-get count of frames, fifo errors and dma errors */
struct fgrab_counts {
	u_long fifo_errors;	/* count of fifo errors since open */
	u_long dma_errors;	/* count of dma errors since open */
	u_long frames_captured;	/* count of frames captured since open */
	u_long even_fields_captured; /* count of even fields captured */
	u_long odd_fields_captured; /* count of odd fields captured */
} ;

/* structure to get the frame offsets in memory */
struct fgrab_frame_offset {
  u_long frame_offset[MAX_NUM_FRAMES];
  u_long fb_size;
  u_long mem_off;
  u_long frame_size;
} ;

/* structure for getting and setting direct transfers to vram */
struct fgrab_video {
	u_long	addr;	/* Address of location to dma to */
	u_long	width;	/* Width of memory area */
	u_long	banksize;	/* Size of Vram bank */
	u_long	ramsize;	/* Size of Vram */
};

struct fgrab_capabilities {
        unsigned ioctl;
        unsigned function;
} ;

/* structure for per-frame data that comes at the end of the image	*/
#define FGRAB_EVENT_END_ODD	0x1
#define FGRAB_EVENT_END_EVEN	0x2
#define FGRAB_EVENT_START_FIELD 0x4
#define FGRAB_MAX_EVENT_CNT     4
struct fgrab_per_frame {
        /* 
         * frame_end needs to be first element in this structure to avoid breaking
         * legacy meteor code which assumes that the only thing after the
         * image buffer is a single timeval structure.  RWF 25JUN97
         */
        struct timeval frame_end; 
	unsigned event_cnt;
	struct fgrab_event {
	    struct timeval ts;
	    unsigned flag;
	    } event[FGRAB_MAX_EVENT_CNT];
	unsigned status;
	unsigned index;
        unsigned fifo_errors;
        unsigned dma_errors;
} ;

#define FGRABCAPTUR _IOW('x', 1, int)			 /* capture a frame */
#define FGRABCAPFRM _IOW('x', 2, struct fgrab_capframe)  /* sync capture */
#define FGRABSETGEO _IOW('x', 3, struct fgrab_geomet)  /* set geometry */
#define FGRABGETGEO _IOR('x', 4, struct fgrab_geomet)  /* get geometry */
#define FGRABSHUE   _IOW('x', 6, signed char)		/* set hue */
#define FGRABGHUE   _IOR('x', 6, signed char)		/* get hue */
#define FGRABSFMT   _IOW('x', 7, unsigned long)	/* set format */
#define FGRABGFMT   _IOR('x', 7, unsigned long)	/* get format */
#define FGRABSINPUT _IOW('x', 8, unsigned long)	/* set input dev */
#define FGRABGINPUT _IOR('x', 8, unsigned long)	/* get input dev */
#define	FGRABSCHCV  _IOW('x', 9, unsigned char)	/* set uv gain */
#define	FGRABGCHCV  _IOR('x', 9, unsigned char)	/* get uv gain */
#define	FGRABSCOUNT _IOW('x',10, struct fgrab_counts)
#define	FGRABGCOUNT _IOR('x',10, struct fgrab_counts)
#define FGRABSFPS   _IOW('x',11, unsigned short)	/* set fps */
#define FGRABGFPS   _IOR('x',11, unsigned short)	/* get fps */
#define FGRABSSIGNAL _IOW('x', 12, unsigned int)	/* set signal */
#define FGRABGSIGNAL _IOR('x', 12, unsigned int)	/* get signal */
#define	FGRABSBRIG  _IOW('x', 13, unsigned char)	/* set brightness */
#define FGRABGBRIG  _IOR('x', 13, unsigned char)	/* get brightness */
#define	FGRABSCSAT  _IOW('x', 14, unsigned char)	/* set chroma sat */
#define FGRABGCSAT  _IOR('x', 14, unsigned char)	/* get uv saturation */
#define	FGRABSCONT  _IOW('x', 15, unsigned char)	/* set contrast */
#define	FGRABGCONT  _IOR('x', 15, unsigned char)	/* get contrast */
#define FGRABSHWS   _IOW('x', 16, unsigned char)	/* set hor start reg */
#define FGRABGHWS   _IOR('x', 16, unsigned char)	/* get hor start reg */
#define FGRABSVWS   _IOW('x', 17, unsigned char)	/* set vert start reg */
#define FGRABGVWS   _IOR('x', 17, unsigned char)	/* get vert start reg */
#define	FGRABSTS    _IOW('x', 18, unsigned long)	/* set time stamp */
#define	FGRABGTS    _IOR('x', 18, unsigned long)	/* get time stamp */
#define	FGRABGFROFF _IOR('x', 19, struct fgrab_frame_offset ) /* get frame offsets */
#define	FGRABCKCAP  _IOW('x', 20, struct fgrab_capabilities ) /* check capabilities  */
#define FGRABSPCLOCK _IOW('x', 21, unsigned long)	/* set pixel clock	*/
#define FGRABGPCLOCK _IOR('x', 21, unsigned long)	/* get pixel clock	*/
				/* FGRABCAPTUR capture options */
#define FGRAB_CAP_SINGLE	0x0001	/* capture one frame */
#define FGRAB_CAP_CONTINOUS	0x0002	/* continuously capture */
#define FGRAB_CAP_STOP_CONT	0x0004	/* stop the continuous capture */
#define FGRAB_CAP_CONT_ONCE	0x0008	/* capture all allocated frames once*/

				/* FGRABCAPFRM capture commands */
#define FGRAB_CAP_N_FRAMES	0x0001	/* capture N frames */
#define FGRAB_CAP_STOP_FRAMES	0x0002	/* stop capture N frames */
#define	FGRAB_HALT_N_FRAMES	0x0003	/* halt of capture N frames */
#define FGRAB_CONT_N_FRAMES	0x0004	/* continue after above halt */

				/* valid video input formats:  */
#define FGRAB_FMT_NTSC		0x00100	/* NTSC --  initialized default */
#define FGRAB_FMT_PAL		0x00200	/* PAL */
#define FGRAB_FMT_SECAM		0x00400	/* SECAM */
#define FGRAB_FMT_AUTOMODE	0x00800 /* auto-mode */


#define FGRAB_INPUT_DEV0	0x01000	/* camera input 0 -- default */
#define FGRAB_INPUT_DEV1	0x02000	/* camera input 1 */
#define FGRAB_INPUT_DEV2	0x03000	/* camera input 2 */
#define FGRAB_INPUT_DEV3	0x04000	/* camera input 3 */
#define FGRAB_INPUT_DEV_RGB	0x05000	/* for rgb version of device */
#define FGRAB_INPUT_DEV_SVIDEO	0x06000 /* S-video input port */
#define FGRAB_INPUT_DEV_MASK    0x07000
#define FGRAB_INPUT_EXT_SYNC    0x08000 /* Flag usage of external sync  */

				/* valid video output formats:  */
#define FGRAB_GEO_RGB16		0x0010000 /* packed -- initialized default */
#define FGRAB_GEO_RGB24		0x0020000 /* RBG 24 bits packed */
					  /* internally stored in 32 bits */
#define FGRAB_GEO_YUV_PACKED	0x0040000 /* 4-2-2 YUV 16 bits packed */
#define FGRAB_GEO_YUV_PLANAR	0x0080000 /* 4-2-2 YUV 16 bits planar */
#define FGRAB_GEO_YUV_16       0x20000000	/* YUV 16 format */
#define FGRAB_GEO_YUV_12       0x10000000	/* YUV 12 format */
#define FGRAB_GEO_YUV_9	       0x40000000	/* YUV 9 format */
#define FGRAB_GEO_MONO8	       0x80000000 /* 8 bit/pixel monochrome	*/
#define FGRAB_GEO_OUTPUT_MASK  0xf00f0000
#define FGRAB_GEO_UNSIGNED	0x0400000 /* unsigned uv outputs */

#define FGRAB_GEO_INTERLACE	0x0000000
#define FGRAB_GEO_EVEN_ONLY	0x1000000 /* set for even only field capture */
#define FGRAB_GEO_ODD_ONLY	0x2000000 /* set for odd only field capture */
#define FGRAB_GEO_ALTERNATE	0x3000000
#define FGRAB_GEO_NON_INTERLACE	0x4000000
#define FGRAB_GEO_FIELD_MASK	0x7000000

#define FGRAB_SIG_MODE_MASK    0xffff0000
#define FGRAB_SIG_FRAME        0x00000000 /* signal every frame */
#define FGRAB_SIG_FIELD        0x00010000 /* signal every field */


/* following structure is used to coordinate the synchronous */
	   
struct fgrab_mem {
		/* kernel write only  */
	int	frame_size;	 /* row*columns*depth */
	unsigned num_bufs;	 /* number of frames in buffer (1-32) */
	u_long  frames_captured; /* total frames captured */
	int     cur_frame;       /* index of most recent acquisition */
	int     cur_field;       /* 0=even, 1=odd */
		/* user and kernel change these */
	int	lowat;		 /* kernel starts capture if < this number */
	int	hiwat;		 /* kernel stops capture if > this number.
				    hiwat <= numbufs */
	unsigned active;	 /* bit mask of active frame buffers
				    kernel sets, user clears */
	int	num_active_bufs; /* count of active frame buffer
				    kernel increments, user decrements */
		/* reference to mmapped data */
	caddr_t	buf;		 /* The real space (virtual addr) */
} ;

#endif /* ifndef _MACHINE_IOCTL_FGRAB_H */

/*
 * Following is from drivers/scsi/hosts.c of 2.0.18.
 * Overrides for Emacs so that we follow Linus's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-indent-level: 4
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -4
 * c-argdecl-indent: 4
 * c-label-offset: -4
 * c-continued-statement-offset: 4
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */
