/*
 *	Copyright (C) 1996  Jim Bray (jb@gandalf.cs.wcu.edu). Code provided
 *      by Jason Lango and Jim Kurien at the AI Lab, and the FreeBSD driver
 *      folks.
 *
 *	This work was supported by RWI (Real World Interface) Inc.,
 *	and the AI Lab at Brown University.
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Version 2:
 *  Cleaned up some cast problems. Changed all the printf()s to perror()s.
 *  Improved signal handling.
 * Version 3:
 *  Renamed from view to vu to avoid name conflict.
 */
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/fcntl.h>
#include <sys/signal.h>
#include <sys/time.h>
#include <sys/mman.h>

#include "vu_xsub.h"
#include "vu_msub.h"

extern int errno;

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <iostream.h>
#include <sched.h>

#define ROWS 480
#define COLS 640

#define SIZE (ROWS * COLS)

#define WINDOW_SIZE_X COLS
#define WINDOW_SIZE_Y ROWS

static char *DevName;
static unsigned char *mmbuf, *fbuf ;

static timeval old;
static int decratio;
static int dsiz_x, dsiz_y;
static unsigned char *decbuf;
static find;

void procframe(unsigned char *buf,struct timeval *tv);

void procframe(unsigned char *buf,struct timeval *tv)
{
    struct timezone tz = { 0,0 } ;
    static struct timeval tlist[100],slist[100],flist[100];
    tlist[find] = *tv;
    gettimeofday(&slist[find],&tz);
    /* directly access the frame buffer array data in mmbuf */
    if (decratio == 0) { }	/* this means skip dissplay altogether	*/
    else if (decratio == 1) {
	putframe(buf);
    } else if (decratio == 2) {
	unsigned char *decptr = decbuf ;
	int x,y;
	/* this code assumes the x dimension is a multiple of 4		*/
	int *ibuf = (int *)buf;
	for (y=0;y<WINDOW_SIZE_Y;y+=2) {
	    for (x=0;x<(WINDOW_SIZE_X);x+=4) {
		int yoff, sum;
		register int mask = 0x00ff00ff;
		register int temp;
		temp=*ibuf;
		sum = (temp&mask) + ( (temp>>8) & mask ) ;
		temp=ibuf[WINDOW_SIZE_X/4];
		sum += (temp&mask) + ( (temp>>8) & mask ) ;
		*decptr++ = ( (sum&0x03ff0000) >> 18 ) ;
		*decptr++ = ( (sum&0x03ff) >> 2 ) ;
		ibuf++ ;
	    }
	    ibuf += (WINDOW_SIZE_X/4) ;
	}
	putframe(decbuf);
    } else {	/* iff decratio is not 1 it must be 4	*/
	unsigned char *decptr = decbuf ;
	int x,y;
	/* this code assumes the x dimension is a multiple of 4		*/
	int *ibuf = (int *)buf;
	for (y=0;y<WINDOW_SIZE_Y;y+=4) {
	    for (x=0;x<(WINDOW_SIZE_X/4);x++) {
		int i,yoff;
		int sum = 0;
		register int mask = 0x00ff00ff;
		for (i=0,yoff=y*(WINDOW_SIZE_X/4)+x; i<4;i++,yoff+=(WINDOW_SIZE_X/4)) {
		    register int temp=ibuf[yoff];
		    sum += (temp&mask) + ( (temp>>8) & mask ) ;
		}
		*decptr++ = ( (sum&0x07ff) + ( (sum>>16) & 0x07ff) ) >>4 ;
	    }
	}
	putframe(decbuf);
    }
    gettimeofday(&flist[find],&tz);
    find++;
    if (find>=100) {
	int i;
	fprintf(stdout,"%9.3f\n", 
		(tlist[0].tv_sec-old.tv_sec) + (1.e-6 * (tlist[0].tv_usec-old.tv_usec)));
	fprintf(stdout,"%9.3f %9.3f\n", 
		    (slist[0].tv_sec-tlist[0].tv_sec) + (1.e-6 * (slist[0].tv_usec-tlist[0].tv_usec)),
		    (flist[0].tv_sec-tlist[0].tv_sec) + (1.e-6 * (flist[0].tv_usec-tlist[0].tv_usec)));
	for (i=1;i<100;i++) {
	    fprintf(stdout,"%9.3f %9.3f %9.3f\n", 
		    (slist[i].tv_sec-tlist[i].tv_sec) + (1.e-6 * (slist[i].tv_usec-tlist[i].tv_usec)),
		    (flist[i].tv_sec-tlist[i].tv_sec) + (1.e-6 * (flist[i].tv_usec-tlist[i].tv_usec)),
		    (tlist[i].tv_sec-tlist[i-1].tv_sec) + (1.e-6 * (tlist[i].tv_usec-tlist[i-1].tv_usec))
		    );
	}
	if (decratio != 0) closedownDisplay();
	exit(0);
    }
}

void main(int argc, char **argv)
{

    struct sched_param sched_param ;
    struct timezone tz = { 0,0 } ;
    char *DevName;
    int i,c;
    int nbuf;
    int fskip;
    if (argc >1) {
	nbuf = atoi(argv[1]);
    } else {
	nbuf = 1 ;
    }
    if (argc > 2) {
	decratio = atoi(argv[2]) ;
	if ( ! ( (decratio==0) | (decratio==1) | (decratio==2) | (decratio==4) ) ) {
	    fprintf(stderr,"decimation ratio must be 1, 2, or 4\n");
	    exit(1);
	}
    } else {
	decratio = 1 ;
    }
    if (decratio != 0) {
	dsiz_x = WINDOW_SIZE_X/decratio ;
	dsiz_y = WINDOW_SIZE_Y/decratio ;
	if (decratio!=1) {
	    if ( (decbuf = (unsigned char *)malloc(dsiz_x*dsiz_y)) == NULL) {
		fprintf(stderr,"cannot allocate memory for decimation buffer\n");
		exit(1);
	    }
	}
	setupDisplay(dsiz_x, dsiz_y);
    }

    if (argc > 3) {
	fskip = atoi(argv[3]);
    } else {
	fskip = 1;
    }

    if ( argc > 4 )
	DevName = argv[4];
    else
	DevName = FGRAB_DEFAULT_DEV;
    if ( (fgrab_setup(procframe, DevName, WINDOW_SIZE_X,WINDOW_SIZE_Y,nbuf,fskip) ) ) {
	fprintf(stderr,"vu: error seting up meteor\n");
	exit(1);
    }

    find = 0 ;
  
    if (mlockall(MCL_CURRENT|MCL_FUTURE)!=0) {
	fprintf(stderr,"Warning: could not lock pages in memory\n");
    }
    sched_param.sched_priority = 99 ;
    if (sched_setscheduler(0,SCHED_FIFO,&sched_param)!=0) {
	fprintf(stderr,"Warning: cannot set up real time scheduler\n");
    }
    gettimeofday(&old,&tz); 
    if ( (fgrab_start() ) ) {
	fprintf(stderr,"vu: error starting meteor\n");
	exit(1);
    }
    while(1)
	pause();
}

