/*
 *	Copyright (C) 1996  Jim Bray (jb@gandalf.cs.wcu.edu). Code provided
 *      by Jason Lango and Jim Kurien at the AI Lab, and the FreeBSD driver
 *      folks.
 *
 *	This work was supported by RWI (Real World Interface) Inc.,
 *	and the AI Lab at Brown University.
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Version 2:
 *  Cleaned up some cast problems. Changed all the printf()s to perror()s.
 *  Improved signal handling.
 * Version 3:
 *  Renamed from view to vu to avoid name conflict.
 */
#include <sys/types.h>
#include <sys/mman.h>
#define MAP_FAILED ((void *)-1)	/* docs say this should be defined in mman 	*/
#include <sys/fcntl.h>
#include <sys/signal.h>
#include <sys/time.h>
#include <unistd.h>

#include "meteor_ioctl.h"
#include "fgrab_ioctl.h"
#define VU_MSUB_INTERNAL
#include "vu_msub.h"
#undef  VU_MSUB_INTERNAL

extern int errno;

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <iostream.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/extensions/XShm.h>
#include <sys/shm.h>
#include <X11/keysym.h>

/* Record the routine that provides the callback			*/
static void (*procframe)(unsigned char *buf,struct timeval *tv);

static char *DevName;
static unsigned char *mmbuf, **fbuf ;
static struct fgrab_per_frame **pf;
static bufno ;		/* index in sequence of buffers of next frame 	*/
static nbuf ;		/* number of frame buffers     			*/
static void gotframe(int x);
static struct fgrab_mem *mm;
static int fd;	/* file descriptor for meteor device	*/
static    struct fgrab_frame_offset off;
static int packed_flag;


static struct sigaction sigact = 
((struct sigaction){ sa_handler: gotframe,
		     sa_mask: (sigset_t)0,
		     sa_flags: SA_RESTART });

static void gotframe(int x) {
    /* directly access the frame buffer array data in mmbuf */
    while (pf[bufno]->status) {
	if (packed_flag) {
	    int i; 
	    unsigned char *ucbuf=fbuf[bufno]; 
	    for (i=0;i<off.frame_size;i+=2) ucbuf[i>>1] = ucbuf[i+1] ; /* for packed mode */
	}
	if (pf[bufno]->error!=0) {
	    fprintf(stderr,"index %d buffer %d Errors: %s%s%s\n",
		    pf[bufno]->index,bufno,
		    (pf[bufno]->error&FGRAB_ERROR_FIFO) ? "FIFO " : "",
		    (pf[bufno]->error&FGRAB_ERROR_DMA) ? "DMA " : "",
		    (pf[bufno]->error&FGRAB_ERROR_BUS) ? "BUS " : "" );
	}
	procframe(fbuf[bufno],&(pf[bufno]->frame_end));
	pf[bufno]->status = 0 ;
	bufno++;
	if (bufno>=nbuf) bufno = 0 ;
    }
}

int fgrab_setup(void (*procframe_)(unsigned char *buf,struct timeval *tv),
		 char *DevName,
		 unsigned int window_size_x, unsigned int window_size_y,
                 int nbuf_,int fskip) {
    
    struct fgrab_geomet geo;
    struct fgrab_capabilities ck;
    int size = window_size_x * window_size_y ;
    int c;

    /* remember the callback routine		*/
    procframe = procframe_ ;
    /* record number of buffers			*/
    nbuf = nbuf_ ;

    if ((fd = open(DevName, O_RDWR)) < 0) {
	perror("open failed");
	printf("device %s\n", DevName);
	return(1);
    }

    c = 1 ;
    if (ioctl(fd, FGRABSTS, &c) < 0) {
	perror("ioctl SetTime failed");
	return(1);
    }

    ck.ioctl = FGRABSETGEO ;
    ck.function = FGRAB_GEO_MONO8 ;
    if (ioctl(fd, FGRABCKCAP, &ck) >= 0) {
	geo.oformat = FGRAB_GEO_MONO8;
	packed_flag = 0 ;
	fprintf(stderr,"GEO_MONO8 selected\n");
    } else {
	ck.function = FGRAB_GEO_YUV_16 ;
	if (ioctl(fd, FGRABCKCAP, &ck) >= 0) {
	    geo.oformat = FGRAB_GEO_YUV_16;
	    fprintf(stderr,"GEO_YUV_16 selected\n");
	    packed_flag = 0 ;
	} else {
	    geo.oformat = FGRAB_GEO_YUV_PACKED;
	    packed_flag = 1 ;
	    fprintf(stderr,"GEO_YUV_PACKED selected\n");
	}
    }
    geo.rows = window_size_y;
    geo.columns = window_size_x;
    geo.frames = nbuf;
    if (ioctl(fd, FGRABSETGEO, &geo) < 0) {
	perror("ioctl SetGeometry failed");
	return(1);
    }


    if (ioctl(fd, FGRABGFROFF, &off) < 0) {
	perror("ioctl GetFrameOffset failed");
	return(1);
    }


    c = FGRAB_FMT_NTSC;

    if (ioctl(fd, FGRABSFMT, &c) < 0) {
	perror("ioctl SetFormat failed");
	return(1);
    }


    c = FGRAB_INPUT_DEV0;

    if (ioctl(fd, FGRABSINPUT, &c) < 0) {
	perror("ioctl Setinput failed");
	return(1);
    }


    c = 30/fskip ;

    if (ioctl(fd, FGRABSFPS, &c) < 0) {
	perror("ioctl SetFps failed");
	return(1);
    }

    mmbuf=(unsigned char *)mmap((caddr_t)0, off.fb_size, 
				PROT_READ|PROT_WRITE,
				MAP_FILE|MAP_SHARED, fd, (off_t)0);
    if ( mmbuf == MAP_FAILED ) {
	perror("mmap failed");
	return(1);
    }
    mm = (struct fgrab_mem *)(mmbuf + off.mem_off) ;
    if ( ( (fbuf=(unsigned char  **)malloc(nbuf*sizeof(unsigned char  *))) == NULL) ||
         ( (pf  =(struct fgrab_per_frame **)malloc(nbuf*sizeof(struct fgrab_per_frame *))) == NULL) ) {
	fprintf(stderr,"vu_msub: setup failed to allocate memory\n");
	return(1);
    }
    for (bufno=0;bufno<nbuf;bufno++) {
	fbuf[bufno] = mmbuf + off.frame_offset[bufno];
	pf[bufno] = (struct fgrab_per_frame *)(fbuf[bufno]+off.frame_size);
    }
    bufno = 0 ;

    if ( sigaction(SIGUSR2, &sigact, NULL) ) {
	perror("sigaction failed");
	return(1);
    }


    c = SIGUSR2;
    if (ioctl(fd, FGRABSSIGNAL, &c) < 0) {
	perror("ioctl SetSignal failed");
	return(1);
    }

    return(0);
}

int fgrab_start(void) {
    int c;
  
    if (nbuf==1) {
	/* continuous frame capture */
	c = FGRAB_CAP_CONTINOUS ;
	if (ioctl(fd, FGRABCAPTUR, &c)) {
	    perror("ioctl CaptContinuous failed");
	    return(1);
	}
    } else {
	/* continuous frame capture */
	{
	    struct fgrab_capframe capfrm;
	    capfrm.lowat=capfrm.hiwat = -1 ;
	    capfrm.command = FGRAB_CAP_N_FRAMES ;
	    if (ioctl(fd, FGRABCAPFRM, &capfrm)) {
		perror("ioctl CaptFrame failed");
		return(1);
	    }
	}
    }

    return(0);
}
