/*
 *	Copyright (C) 1996  Jim Bray (jb@gandalf.cs.wcu.edu). Code provided
 *      by Jason Lango and Jim Kurien at the AI Lab, and the FreeBSD driver
 *      folks.
 *
 *	This work was supported by RWI (Real World Interface) Inc.,
 *	and the AI Lab at Brown University.
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Version 2:
 *  Cleaned up some cast problems. Changed all the printf()s to perror()s.
 *  Improved signal handling.
 * Version 3:
 *  Renamed from view to vu to avoid name conflict.
 */
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/fcntl.h>
#include <sys/signal.h>

extern int errno;

#define VU_XSUB_INTERNAL
#include "vu_xsub.h"
#undef  VU_XSUB_INTERNAL

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <iostream.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/extensions/XShm.h>
#include <sys/shm.h>
#include <X11/keysym.h>

static Display* display_;
static GC gc_;
static Window wtwin_;

static XImage *shmimage;
static XShmSegmentInfo* shminfo;

static unsigned int lookup[256];
static int maxcols = 256;
static char *DevName;
static char *mmbuf, *fbuf;

static int size_x;
static int size_y;
static int size;

void putframe(unsigned char *buf)
{
  /* directly access the frame buffer array data in mmbuf */
  int pix;
  int cnt = size ;
  unsigned char *to, *from ;

  for (pix=0,to=(unsigned char*)shmimage->data,from=buf;
       pix<cnt;pix++) *to++ = lookup[*from++] ;
  // Draw screen onto display
  XShmPutImage(display_, wtwin_, gc_, shmimage,
	       0, 0, 0, 0, size_x, size_y, False);
  XSync(display_, 0);
}

void setupDisplay(unsigned int window_size_x,
			   unsigned int window_size_y) {
  XGCValues gc_values;
  size_x = window_size_x;
  size_y = window_size_y;
  size = size_x * size_y ;
  display_ = XOpenDisplay(NULL);
  wtwin_ = XCreateSimpleWindow(display_, DefaultRootWindow(display_),
			       0, 0, window_size_x, window_size_y, 2,
			       BlackPixel(display_, DefaultScreen(display_)),
			       BlackPixel(display_, DefaultScreen(display_)));  
  XMapWindow(display_, wtwin_);
  XSelectInput(display_, wtwin_,KeyPressMask | KeyReleaseMask);
  gc_values.graphics_exposures = False;
  gc_ = XCreateGC(display_, wtwin_, GCGraphicsExposures, &gc_values);  

  //SHARED MEMORY PORTION
  
  int depth;
  int screen;
  XWindowAttributes win_attributes;

  screen = DefaultScreen(display_);
  XGetWindowAttributes(display_, wtwin_, &win_attributes);
  depth = win_attributes.depth;
  
  shminfo = (XShmSegmentInfo*) malloc (sizeof(XShmSegmentInfo));
  shmimage = XShmCreateImage(display_, DefaultVisual(display_, screen),
			     depth, ZPixmap, NULL, shminfo, window_size_x,
			     window_size_y);
  shminfo->shmid = shmget(ftok(DevName, 'v'),
			  shmimage->bytes_per_line * shmimage->height,
			  IPC_CREAT | 0777);
  shminfo->shmaddr = (char *) shmat(shminfo->shmid, (void *) 0, 0);
  shmimage->data = shminfo->shmaddr;
  
  XShmAttach(display_, shminfo);
  // Doing a delete here assures that if we die we will release the shared
  // memory.  Since we did an attach above it will stay around till we quit.
  shmctl(shminfo->shmid,IPC_RMID,NULL);

  //END SHARED MEMORY PORTION

  // Set palette

  XVisualInfo desired_visual_template;
  XVisualInfo *visual_info_list; 
  int no_visual_matched;
  Colormap wtcolormap;
  int pixelNum;

  desired_visual_template.screen = screen;
  desired_visual_template.c_class = PseudoColor;
  visual_info_list = XGetVisualInfo(display_, 
				    VisualScreenMask | VisualClassMask,
				    &desired_visual_template,
				    &no_visual_matched);
  if (no_visual_matched == 0)
    cerr << "No visual matched" << endl;
  
  wtcolormap = DefaultColormap(display_,screen);
  for (pixelNum=0;pixelNum<maxcols;pixelNum++) {
    XColor xcolor ;
    int grey = ((pixelNum/4)*4+2)<<8 ;
    xcolor.red = xcolor.green = xcolor.blue = grey ;
    if (!XAllocColor (display_, wtcolormap, &xcolor)) {
	fprintf(stderr,"cannot allocate quantized grey level %d\n",grey);
	exit(1);
	}
    lookup[pixelNum] = xcolor.pixel ;
  }

}

void closedownDisplay(void) {
shmdt((char *)shminfo->shmaddr);
}
