/*
 *	ioctl functions that are specific to the dt315[25] boards
 *    
 *    See file COPYING for copyright info
 *
 */

/*
 * DT315XSGEXP and DT315XGGEXP give the user acess to an analog gain that is unique to the 
 * dt3152.  This gain has four settings that increment in powers of 2 from .5 to 4. 
 * It is initialized to unity which whould suffice for most purposes.  Software that uses
 * this will not be portable to other devices.  If you find you really need this you
 * might want to consider a change to the generic driver to make the contrast ioctl's
 * be more than 8 bits (use 12 of those bits for this device -- shift right till only
 * 8 bits are non-zero -- that becomes the contrast value to the d-to-a in the bt252.
 * The number of bits shifted becomes the gain exponent.  This ioctl is here just to
 * cover the feature - hopefully it will not be widely used. RWF 4jul97
 */
#define	DT3152SGEXP  	_IOW( 'x', 64+1, unsigned long)       	/* set the 3152 gain exponent	*/
#define	DT3152GGEXP  	_IOR( 'x', 64+2, unsigned long)       	/* get the 3152 gain exponent	*/
/*
 * DT315XGCAPT, DT3152GHFMTMEM, DT3152GHFMTMEM, DT3152GADLUT, and DT3152GPMLUT 
 * are really meant as debugging aids - they should not be used in functional code.
 */
#define	DT315XGCAPT  	_IOR( 'x', 64+3, unsigned long)       	/* get saa7116 capt_ctl (0x40)	*/
#define DT3152GCSR4	_IOR( 'x', 64+4, unsigned long)		/* get 3152 csr4		*/
#define	DT3152GHFMTMEM  _IOWR('x', 64+5, unsigned long)		/* get horizontal format memory */
#define	DT3152GVFMTMEM  _IOWR('x', 64+6, unsigned long)		/* get vertical format memory 	*/
#define	DT3152GADLUT    _IOWR('x', 64+7, unsigned long)		/* get ad lut                 	*/
#define	DT3152GPMLUT    _IOWR('x', 64+8, unsigned long)		/* get pm lut                 	*/

#define	DT315X_STATUS_XX	0x0000	/* There should be a series of things for status ioctl return	*/
